<?php

/*
***************************************
*
* PrestaShop - Open Source eCommerce Solution
* @link:               www.prestashop.com
*
* @script name:        PrestaShop TrustPay Payment
* @purpose:            Module for accepting payments by TrustPay multi-payment gateway (member of Meracrest group)
* @type:               Payment module
* @author:             Gabriel Schwardy ( addons@caleydon.com )
* @copyright:          (c) 2001-2020 Caleydon s.r.o. ( www.caleydon.com )
*
***************************************
*/

/*
 * NOTIFICATION.PHP - Receiving the notifiaction about payment from TrustPay
 * The script changing order statuses based on input HTTP variables.
 * DOCS TrustPay: https://doc.trustpay.eu/?php#payment-notifications
 * DOCS TrustCard: https://doc.trustpay.eu/?php#cards-notification
*/

/*

TRUSTPAY NOTIFICATION URL:
notification.php
?AccountId=2107111111
&Type=CRDT
&Amount=6.00
&Currency=EUR
&ResultCode=0
&PaymentId=177561
&Reference=1234567890
&CounterAccount=SK3399520000002107425307
&CounterAccountName=TestAccount
&Signature=C4FFA5F7B57A0F5E6DD105197C8B7D7B9214509F3618CDC0F527C9B4E57F0334

TRUSTCARD NOTIFICATION URL:
notification.php
?AccountId=2107111111
&Type=CRDT
&Amount=6.00
&Currency=EUR
&Reference=1234567890
&ResultCode=0
&PaymentRequestId=177561
&Signature=C4FFA5F7B57A0F5E6DD105197C8B7D7B9214509F3618CDC0F527C9B4E57F0334
&CardId=8a8294495c589aa3015c63bd6d237e46
&CardMask=0162
&CardExpiration=1020
&AuthNumber=198159

*/

include(dirname(__FILE__).'/../../../config/config.inc.php');
//include(dirname(__FILE__).'/../../../init.php');
include(dirname(__FILE__).'/../../../header.php');
require_once(dirname(__FILE__).'/../trustpay.php');

$errors = '';
$trustpay = new Trustpay();

$debug = Configuration::get('TRUSTPAY_DEBUG');

/* 1. BUILDING VARIABLES FROM RETURN URL */
/* IMPORTANT: DO NOT USE (int)($_GET['VALUE']) BECAUSE IT REMOVE 0 FROM THE STRING BEGINING! */

/* Povinne parametre */
$tp_key = Configuration::get('TRUSTPAY_PWD'); // Get merchant's Secret key from DB
$tp_aid = Tools::safeOutput(Tools::getValue('AccountId')); // Merchant account ID / Numeric(10)
$tp_typ = Tools::safeOutput(Tools::getValue('Type')); // Type of transaction - CRDT or DBIT / Char(4)
$tp_amt = Tools::safeOutput(Tools::getValue('Amount')); // Amount of the payment - Max. 2 decimal places
$tp_cur = Tools::safeOutput(Tools::getValue('Currency')); // Currency of the payment - e.g. EUR / Char(3)
$tp_ref = Tools::getValue('Reference'); // Merchants payment identification (order ID) / Varchar(35)
$tp_res = Tools::getValue('ResultCode'); // Result code / Numeric(4)
$tp_pid = Tools::getValue('PaymentId'); // Payment ID / Numeric(10)
$tp_sig = Tools::getValue('Signature'); // Signature / Char(64)

/* Nepovinne parametre - Wire payments */
$tp_oid = Tools::getValue('OrderId'); // TrustPay Order ID (ID of payment order) / Numeric(10)
$tp_ca = Tools::getValue('CounterAccount'); // Counter account / Varchar(34)
$tp_can = Tools::getValue('CounterAccountName'); // Counter account name / Varchar(140)

/* Nepovinne parametre - Card payments */
$tp_prid = Tools::getValue('PaymentRequestId'); // Payment request ID (sent when available, can be used for inquiries regarding failed payments) / Numeric(10)
$tp_cardid = Tools::getValue('CardId'); // Card ID. This can be used to make card-on-file payments / Varchar(64)
$tp_cardmask = Tools::getValue('CardMask'); // Masked card number or last 4 digits of the card / Varchar(19)
$tp_cardexp = Tools::getValue('CardExpiration'); // Card expiration in MMYY format / Number(4)
$tp_authnumber = Tools::getValue('AuthNumber'); // Authorization number / Varchar(7)

/* 

/modules/trustpay/response/notification.php?AccountId=2107730421&Type=CRDT&Amount=5.45&Currency=EUR&ResultCode=0&PaymentId=11119921&Reference=&CounterAccount=SK6299520000002107770784&CounterAccountName=Test+account&Signature=21E59331208F95FCA208E28611751CCD04C98BD29D38A8E7C01323FF5037556C

/modules/trustpay/response/notification.php?AccountId=2107730421&Type=CRDT&Amount=27.09&Currency=EUR&ResultCode=0&PaymentId=11119973&Reference=12&CounterAccount=SK6299520000002107770784&CounterAccountName=Test+account&Signature=A487D1A6DBD275481C6448C72F2646AC2BCB645DCDE7BE614102E1655F951905

/modules/trustpay/response/notification.php?AccountId=2107730421&Type=CRDT&Amount=9.39&Currency=EUR&Reference=13&ResultCode=3&PaymentRequestId=4032&Signature=06D71B588A54AF359AA000EFA106C49E887BB226266E950905AB7631959C0E7B&CardID&CardMask=0000&CardExpiration=1121&AuthNumber=435334&CardRecTxSec&AcqResId=000.000.000

*/

    if (isset($tp_ref) && $tp_ref == '') {
        /* Return code 200 & stop the script */
        var_dump(http_response_code());
        echo '<br>';
        exit("REF neexistuje");
    } else {
        $order_exist = $trustpay->_getOrderIdExist($tp_ref);
        if (isset($order_exist) && $order_exist == '0') {
            http_response_code(500);
        } else {
            // echo $order_exist;
        }
    }

    /* 2A. LOAD INFO ABOUT AN EXISTING ORDER */
    // Get ID cart from ID order
    $cart = new Cart((int)($tp_ref));
    $total = (float)(number_format($cart->getOrderTotal(true, 3), 2, '.', ''));
    $customer = new Customer((int)$cart->id_customer);

    $payment_type = '0';

    /* 3. CREATE SIGNATURE */
    $message = sprintf("%d/%s/%s/%s/%d", $trustpay->AID, $total, $tp_cur, $tp_ref, $payment_type);
    $signature = $trustpay->GetSignature(Configuration::get('TRUSTPAY_PWD'), $message);

    /* 5. CHECK THE VALIDITY AND MATCH OF VARIABLES */
    if (!isset($tp_aid)) {
        $errors .= $trustpay->getL('msg_aid').'<br>';
    }

    if (!isset($tp_typ)) {
        $errors .= $trustpay->getL('msg_typ').'<br>';
    }

    if ($tp_typ == 'DBIT') {
        $errors .= $trustpay->getL('msg_dbit').'<br>'; 
    }

    if ($tp_typ != 'CRDT' AND $tp_typ != 'DBIT') {
        $errors .= $trustpay->getL('msg_crdt').'<br>'; 
    }

    if ($total <= '0') {
        $errors .= $trustpay->getL('msg_total').'<br>';
    }

    if (!isset($tp_cur)) {
        $errors .= $trustpay->getL('msg_cur').'<br>';
    }

    if (!isset($tp_ref)) {
        $errors .= $trustpay->getL('msg_ref').'<br>';
    }

    if (!isset($tp_pid)) {
        $errors .= $trustpay->getL('msg_pid').'<br>'; 
    }

    if (!isset($tp_oid)) {
//      $errors .= $trustpay->getL('msg_oid').'<br>';
    }

/*
    if (!isset($tp_sig)) {
        $errors .= $trustpay->getL('msg_sign').'<br>';
    } elseif ($signature != $tp_sig) {
        $errors .= $trustpay->getL('msg_signs').'<br>';
    }
*/

/* 6. BUILD PRIVATE MESSAGES (PM) */

/* 6.1 PM for TrustPay payment info */
$private_message = '<br>'.$trustpay->getL('msg_paymentinfo').'<br>';
$private_message .= $trustpay->getL('msg_pidinfo').': '.$tp_pid.'<br>';
$private_message .= $trustpay->getL('msg_oidinfo').': '.$tp_oid;

/* 6.2 PM for Card payment info */
/*
$private_message .= '<br><br>'.$trustpay->getL('msg_cardinfo').'<br>';
$private_message .= $trustpay->getL('msg_cardid').': '.$tp_cardid.'<br>';
$private_message .= $trustpay->getL('msg_cardmask').': '.$tp_cardmask.'<br>';
$private_message .= $trustpay->getL('msg_cardexp').': '.$tp_cardexp.'<br>';
$private_message .= $trustpay->getL('msg_authnumber').': '.$tp_authnumber.'<br>';
$private_message .= $trustpay->getL('msg_acqresid').': '.$tp_acqresid;
*/

/* 7. SET ASSOCIATED ORDER STATUS BASED ON RES (tp_res) VALUE */
    if (empty($errors))
    {

        /*************************** 6.1 SUCCESS STATUS ***************************/
        if ($tp_res == '0') 
        {
                /*
                SUCCESS [redirect, email notification, http notification]
                Payment was successfully processed.
                When received in notification, funds have been credited to the merchant account.
                Merchant can provide goods or services without delay.
                NOTE: Result codes 0, 3 or 4 can be treated as a successfully executed payment.
                */

                /* If received amount is greater or equal -----> Update order status to 'Payment received' */
                if ($tp_amt >= $total) {

                    $tss_msg = $trustpay->getL('msg_transaction').'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_tssno').'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_required').' '.$total.'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_received').' '.$tp_amt.'<br>';

                    $history = new OrderHistory();
                    $history->id_order = (int)($order_exist);
                    $history->changeIdOrderState((int)Configuration::get('TRUSTPAY_STATUS_OK'), $history->id_order);
                    $history->addWithemail();
                } 
                // If received amount is less -----> Update order status to 'Error'
                else {

                    $tss_msg = $trustpay->getL('msg_tssno').'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_required').' '.$total.'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_received').' '.$tp_amt.'<br>';

                    $history = new OrderHistory();
                    $history->id_order = (int)($order_exist);
                    $history->changeIdOrderState((int)Configuration::get('TRUSTPAY_STATUS_FAIL'), $history->id_order);
                    $history->addWithemail();
                }

            /* Add the private order message to an order */
            $trustpay->_addNewPrivateMessage((int)$order_exist, $tss_msg.$private_message);

        }    /* end of tp_res == 0 */

        /*************************** 6.2 PENDING STATUS (OFFLINE PAYMENT) ***************************/
        elseif ($tp_res == '1') 
        {
                /*
                PENDING [redirect]
                Payment is pending (offline payment)
                */

                /* If received amount is greater or equal -----> Update order status to 'Payment received' */
                if ($tp_amt >= $total) {

                    $tss_msg = $trustpay->getL('msg_transaction'); 
                    $tss_msg .= $trustpay->getL('msg_tssno').'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_required').' '.$total.'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_received').' '.$tp_amt.'<br>';

                    $history = new OrderHistory();
                    $history->id_order = (int)($order_exist);
                    $history->changeIdOrderState((int)Configuration::get('TRUSTPAY_STATUS_PENDING'), $history->id_order);
                    $history->addWithemail();
                } 
                /* If received amount is less -----> Update order status to 'Error' */
                else {

                    $tss_msg = $trustpay->getL('msg_tssno').'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_required').' '.$total.'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_received').' '.$tp_amt.'<br>';

                    $history = new OrderHistory();
                    $history->id_order = (int)($order_exist);
                    $history->changeIdOrderState((int)Configuration::get('TRUSTPAY_STATUS_FAIL'), $history->id_order);
                    $history->addWithemail();
                }

            /* Add the private order message to an order */
            $trustpay->_addNewPrivateMessage((int)$order_exist, $tss_msg.$private_message);

        }    /* end of tp_res == 1 */

        /*************************** 6.3 ANNOUNCED STATUS ***************************/
        elseif ($tp_res == '2') 
        {
                /*
                ANNOUNCED [email notification, http notification]
                TrustPay has been notified that the client placed a payment order or has made payment, 
                but further confirmation from 3rd party is needed. Another notification (with result code 0 - success) 
                will be sent when TrustPay receives and processes payment from 3rd party.
                Funds have not been credited to the merchant account and there is no guarantee they will be.
                */

                /* If received amount is greater or equal -----> Update order status to 'Payment received' */
                if ($tp_amt >= $total) {

                    $tss_msg = $trustpay->getL('msg_announced'); 
                    $tss_msg .= $trustpay->getL('msg_tssno').'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_required').' '.$total.'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_received').' '.$tp_amt.'<br>';

                    $history = new OrderHistory();
                    $history->id_order = (int)($order_exist);
                    $history->changeIdOrderState((int)Configuration::get('TRUSTPAY_STATUS_ANNOUNCED'), $history->id_order);
                    $history->addWithemail();
                } 
                /* If received amount is less -----> Update order status to 'Error' */
                else {
                
                    $tss_msg = $trustpay->getL('msg_tssno').'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_required').' '.$total.'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_received').' '.$tp_amt.'<br>';

                    $history = new OrderHistory();
                    $history->id_order = (int)($order_exist);
                    $history->changeIdOrderState((int)Configuration::get('TRUSTPAY_STATUS_FAIL'), $history->id_order);
                    $history->addWithemail();
                }
                    
            /* Add the private order message to an order */
            $trustpay->_addNewPrivateMessage((int)$order_exist, $tss_msg.$private_message);

        }    /* end of tp_res == 2 */

        /*************************** 6.4 AUTHORIZED STATUS ***************************/
        elseif ($tp_res == '3') 
        {
                /*
                AUTHORIZED [email notification, http notification]
                Payment was successfully authorized. Another notification (with result code 0 - success)
                will be sent when TrustPay receives and processes payment from 3rd party. For card payments 
                (see Appendix IX), funds will be credited to the merchant account of the merchant in a bulk payment 
                on the next settlement day. For other payments, funds will be credited to the merchant account, 
                at a later date. Merchant can provide goods or services without delay.
                NOTE: Result codes 0, 3 or 4 can be treated as a successfully executed payment.
                */

                /* If received amount is greater or equal -----> Update order status to 'Payment received' */
                if ($tp_amt >= $total) {

                    $tss_msg = $trustpay->getL('msg_authorized'); 
                    $tss_msg .= $trustpay->getL('msg_tssno').'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_required').' '.$total.'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_received').' '.$tp_amt.'<br>';

                    $history = new OrderHistory();
                    $history->id_order = (int)($order_exist);
                    $history->changeIdOrderState((int)Configuration::get('TRUSTPAY_STATUS_AUTHORIZED'), $history->id_order);
                    $history->addWithemail();
                } 
                /* If received amount is less -----> Update order status to 'Error' */
                else {
                
                    $tss_msg = $trustpay->getL('msg_tssno').'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_required').' '.$total.'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_received').' '.$tp_amt.'<br>';
                    
                    $history = new OrderHistory();
                    $history->id_order = (int)($order_exist);
                    $history->changeIdOrderState((int)Configuration::get('TRUSTPAY_STATUS_FAIL'), $history->id_order);
                    $history->addWithemail();
                }
                    
            /* Add the private order message to an order */
            $trustpay->_addNewPrivateMessage((int)$order_exist, $tss_msg.$private_message);

        }    /* end of tp_res == 3 */

        /*************************** 6.5 PROCESSING STATUS ***************************/
        elseif ($tp_res == '4') 
        {
                /*
                PROCESSING [email notification, http notification]
                TrustPay has received the payment, but it must be internally processed 
                before it is settled on the merchants account. When the payment is successfully processed, 
                another notification (with the result code 0 - success) will be sent.
                Funds will be credited to the merchant account, at a later date.
                NOTE: Result codes 0, 3 or 4 can be treated as a successfully executed payment.
                */
        
                /* If received amount is greater or equal -----> Update order status to 'Payment received' */
                if ($tp_amt >= $total) {

                    $tss_msg = $trustpay->getL('msg_processing'); 
                    $tss_msg .= $trustpay->getL('msg_tssno').'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_required').' '.$total.'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_received').' '.$tp_amt.'<br>';

                    $history = new OrderHistory();
                    $history->id_order = (int)($order_exist);
                    $history->changeIdOrderState((int)Configuration::get('TRUSTPAY_STATUS_PROCESSING'), $history->id_order);
                    $history->addWithemail();
                } 
                /* If received amount is less -----> Update order status to 'Error' */
                else {
                
                    $tss_msg = $trustpay->getL('msg_tssno').'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_required').' '.$total.'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_received').' '.$tp_amt.'<br>';

                    $history = new OrderHistory();
                    $history->id_order = (int)($order_exist);
                    $history->changeIdOrderState((int)Configuration::get('TRUSTPAY_STATUS_FAIL'), $history->id_order);
                    $history->addWithemail();
                }
                    
            /* Add the private order message to an order */
            $trustpay->_addNewPrivateMessage((int)$order_exist, $tss_msg.$private_message);

        }    /* end of tp_res == 4 */
        
        /*************************** 5.6 AUTHORIZED ONLY STATUS ***************************/
        elseif ($tp_res == '5') 
        {
                /*
                AUTHORIZEDONLY [redirect, email notification, http notification]
                Reserved for future use!
                Card payment was successfully authorized, but not captured. 
                Subsequent MAPI call(s) is (are) required to capture payment.
                */

                /* If received amount is greater or equal -----> Update order status to 'Payment received' */
                if ($tp_amt >= $total) {

                    $tss_msg = $trustpay->getL('msg_authorizedonly'); 
                    $tss_msg .= $trustpay->getL('msg_tssno').'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_required').' '.$total.'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_received').' '.$tp_amt.'<br>';

                    $history = new OrderHistory();
                    $history->id_order = (int)($order_exist);
                    $history->changeIdOrderState((int)Configuration::get('TRUSTPAY_STATUS_AUTHORIZEDONLY'), $history->id_order);
                    $history->addWithemail();
                } 
                /* If received amount is less -----> Update order status to 'Error' */
                else {

                    $tss_msg = $trustpay->getL('msg_tssno').'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_required').' '.$total.'<br>'; 
                    $tss_msg .= $trustpay->getL('msg_received').' '.$tp_amt.'<br>';

                    $history = new OrderHistory();
                    $history->id_order = (int)($order_exist);
                    $history->changeIdOrderState((int)Configuration::get('TRUSTPAY_STATUS_FAIL'), $history->id_order);
                    $history->addWithemail();
                }
                    
            /* Add the private order message to an order */
            $trustpay->_addNewPrivateMessage((int)$order_exist, $tss_msg.$private_message);

        }    /* end of tp_res == 5 */

        elseif ($tp_res == '1001') {
            /*
            1001 - INVALID REQUEST [redirect]
            Data sent is not properly formatted
            */
            $errors .= $trustpay->getL('msg_1001');
        }
        elseif ($tp_res == '1002') {
            /*
            1002 - UNKNOWN ACCOUNT [redirect]
            Account with specified ID was not found. 
            Please check if you are using correct account number - AID.
            */
            $errors .= $trustpay->getL('msg_1002');
        } 
        elseif ($tp_res == '1003') {
            /*
            1003 - MERCHANT ACCOUNT DISABLED [redirect]
            Merchant account has been disabled
            */
            $errors .= $trustpay->getL('msg_1003');
        } 
        elseif ($tp_res == '1004') {
            /*
            1004 - INVALID SIGN [redirect]
            The message is not signed correctly
            */
            $errors .= $trustpay->getL('msg_1004');
        } 
        elseif ($tp_res == '1005') {
            /*
            1005 - USER CANCEL [redirect]
            Customer has cancelled the payment
            */
            $errors .= $trustpay->getL('msg_1005');
        } 
        elseif ($tp_res == '1006') {
            /*
            1006 - INVALID AUTHENTICATION [redirect]
            Request was not properly authenticated
            */
            $errors .= $trustpay->getL('msg_1006');
        } 
        elseif ($tp_res == '1007') {
            /*
            1007 - DISPOSABLE BALANCE [redirect]
            Requested transaction amount is greater than disposable balance
            */
            $errors .= $trustpay->getL('msg_1007');
        } 
        elseif ($tp_res == '1008') {
            /*
            1008 - SERVICE NOT ALLOWED [redirect]
            Service cannot be used or permission to use given service has not been granted.
            If you receive current code, please contact TrustPay for more information.
            */
            $errors .= $trustpay->getL('msg_1008');
        } 
        elseif ($tp_res == '1009') {
            /*
            1009 - PAYSAFECARD TIMEOUT [redirect]
            Cards allocation will be cancelled
            */
            $errors .= $trustpay->getL('msg_1009');
        } 
        elseif ($tp_res == '1010') {
            /*
            1010 - TRANSACTION NOT FOUND [redirect]
            Transaction with specified ID was not found
            */
            $errors .= $trustpay->getL('msg_1010');
        } 
        elseif ($tp_res == '1011') {
            /*
            1011 - UNSUPPORTED TRANSACTION [redirect]
            The requested action is not supported for the transaction
            */
            $errors .= $trustpay->getL('msg_1011');
        } 
        elseif ($tp_res == '1014') {
            /*
            1014 - REJECTED TRANSACTION [client redirects]
            Transaction was rejected by issuer
            */
            $errors .= $trustpay->getL('msg_1014');
        } 
        elseif ($tp_res == '1100') {
            /*
            1100 - GENERAL ERROR [redirect]
            Internal error has occurred
            */
            $errors .= $trustpay->getL('msg_1100');
        }
        elseif ($tp_res == '1101') {
            /*
            1101 - UNSUPPORTED CURRENCY CONVERSION [redirect]
            Currency conversion for requested currencies is not supported
            */
            $errors .= $trustpay->getL('msg_1101');
        }
        else {
            $errors .= $trustpay->getL('msg_noverif');
        }
        
    } /* end of empty(errors) */

    /* If there are any errors - set order status to "Error payment" and add an error private message */
    if (!empty($errors) && isset($order_exist)) {
        $history = new OrderHistory();
        $history->id_order = (int)($order_exist);
        $history->changeIdOrderState((int)Configuration::get('TRUSTPAY_STATUS_FAIL'), $history->id_order);
        $history->addWithemail();

        // Add the private order message to an order
        $trustpay->_addNewPrivateMessage((int)$order_exist, $errors.$private_message);
    }
/* EOF */
