<?php
    
/*
***************************************
*
* PrestaShop - Open Source eCommerce Solution
* @link:               www.prestashop.com
*
* @script name:        PrestaShop TrustPay Payment
* @purpose:            Module for accepting payments by TrustPay multi-payment gateway (member of Meracrest group)
* @type:               Payment module
* @author:             Gabriel Schwardy ( addons@caleydon.com )
* @copyright:          (c) 2001-2020 Caleydon s.r.o. ( www.caleydon.com )
*
***************************************
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class Trustpay extends PaymentModule
{
    public $_html = '';
    public $_postErrors = array();
    
    public $DEBUG; // Debigging mode
    public $AID; // Merchant account id
    public $PAYURL; // Payment URL - TrustPay
    public $PAYURLCARD; // Payment URL - TrustCard

    public $CURR; // Currency
    public $COUNTRY; // Country
    public $LANGUAGE; // Language
    public $DESCRIPTION; // Description

    public $THEME; // PrestaShop theme
    public $TAX_LABEL; // Tax label
    public $ICON; // Icon
    public $BANKICON; // Bank icons
    public $CARDICON; // Cards icons
    public $CMS_ID;

    public $ORDER_METHOD; 
    public $CHECKOUT_TYPE; 
    public $ORDER_ALWAYS; 
    public $tp_os_awaiting; // Waiting for TrustPay domestic payment
    public $tp_os_cardwaiting; // Waiting for card payment

    public $tp_os_pending; // Pending payment status
    public $tp_os_announced; // Announced payment status
    public $tp_os_authorized; // Authorized payment status
    public $tp_os_processing; // Processing payment status
    public $tp_os_authorizedonly; // Authorized only (card payment) payment status

    public $payment_options;
    public $PRIVATE_NOTES;

    public function __construct()
    {
        $this->name = 'trustpay';
        $this->tab = 'payments_gateways';
        $this->version = '1.6.12';
        $this->rdate = '20200505';
        $this->ps_versions_compliancy = array('min' => '1.6.0.0', 'max' => _PS_VERSION_);
        $this->company = 'Caleydon s.r.o.';
        $this->author = 'caleydon.com';
        $this->controllers = array('payment', 'result');

        $this->is_eu_compatible = 1;
        $this->need_instance = 1;

        $this->currencies = true;
        $this->currencies_mode = 'radio';

        $config = Configuration::getMultiple(array(
                                        'TRUSTPAY_DEBUG', 
                                        'TRUSTPAY_AID', 
                                        'TRUSTPAY_PAYURL', 
                                        'TRUSTPAY_PAYURLCARD', 
                                        'TRUSTPAY_CUR', 
                                        'TRUSTPAY_CNT', 
                                        'TRUSTPAY_LNG', 
                                        'TRUSTPAY_DSC', 
                                        'TRUSTPAY_PWD', 
                                        'TRUSTPAY_THEME', 
                                        'TRUSTPAY_TAX_LABEL', 
                                        'TRUSTPAY_ICON', 
                                        'TRUSTPAY_BANKICON', 
                                        'TRUSTPAY_CARDICON', 
                                        'TRUSTPAY_CMS_ID', 
                                        'TRUSTPAY_ORDER_METHOD', 
                                        'TRUSTPAY_CHECKOUT_TYPE', 
                                        'TRUSTPAY_ORDER_ALWAYS', 
                                        'TRUSTPAY_PRIVATE_NOTES', 
                                        'TRUSTPAY_STATUS_AWAITING',
                                        'TRUSTPAY_STATUS_CARDWAITING', 
                                        'TRUSTPAY_STATUS_PENDING', 
                                        'TRUSTPAY_STATUS_ANNOUNCED', 
                                        'TRUSTPAY_STATUS_AUTHORIZED', 
                                        'TRUSTPAY_STATUS_PROCESSING', 
                                        'TRUSTPAY_STATUS_AUTHORIZEDONLY',
                                        'TRUSTPAY_STATUS_OK',
                                        'TRUSTPAY_STATUS_FAIL',
                                        'TRUSTPAY_OS_AWAITING',
                                        'TRUSTPAY_OS_CARDWAITING', 
                                        'TRUSTPAY_OS_PENDING', 
                                        'TRUSTPAY_OS_ANNOUNCED', 
                                        'TRUSTPAY_OS_AUTHORIZED', 
                                        'TRUSTPAY_OS_PROCESSING', 
                                        'TRUSTPAY_OS_AUTHORIZEDONLY', 
        ));

        if (isset($config['TRUSTPAY_DEBUG'])) {
            $this->DEBUG = $config['TRUSTPAY_DEBUG'];
        }

        if (isset($config['TRUSTPAY_AID'])) {
            $this->AID = $config['TRUSTPAY_AID'];
        }

        if (isset($config['TRUSTPAY_PAYURL'])) {
            $this->PAYURL = $config['TRUSTPAY_PAYURL'];
        }

        if (isset($config['TRUSTPAY_PAYURLCARD'])) {
            $this->PAYURLCARD = $config['TRUSTPAY_PAYURLCARD'];
        }

        if (isset($config['TRUSTPAY_CUR'])) {
            $this->CURR = $config['TRUSTPAY_CUR'];
        }

        if (isset($config['TRUSTPAY_CNT'])) {
            $this->COUNTRY = $config['TRUSTPAY_CNT'];
        }

        if (isset($config['TRUSTPAY_LNG'])) {
            $this->LANGUAGE = $config['TRUSTPAY_LNG'];
        }

        $this->TAX_LABEL = Configuration::get('PS_TAX');


        if (isset($config['TRUSTPAY_PWD'])) {
            $this->tp_pwd = $config['TRUSTPAY_PWD'];
        }

        if (isset($config['TRUSTPAY_THEME'])) {
            $this->THEME = $config['TRUSTPAY_THEME'];
        }

        if (isset($config['TRUSTPAY_DSC'])) {
            $this->DESCRIPTION = $config['TRUSTPAY_DSC'];
        }

        if (isset($config['TRUSTPAY_ICON'])) {
            $this->ICON = $config['TRUSTPAY_ICON'];
        }

        if (isset($config['TRUSTPAY_BANKICON'])) {
            $this->BANKICON = $config['TRUSTPAY_BANKICON'];
        }

        if (isset($config['TRUSTPAY_CARDICON'])) {
            $this->CARDICON = $config['TRUSTPAY_CARDICON'];
        }

        if (isset($config['TRUSTPAY_CMS_ID'])) {
            $this->CMS_ID = $config['TRUSTPAY_CMS_ID'];
        }

        if (isset($config['TRUSTPAY_ORDER_METHOD'])) {
            $this->ORDER_METHOD = $config['TRUSTPAY_ORDER_METHOD'];
        }

        if (isset($config['TRUSTPAY_CHECKOUT_TYPE'])) {
            $this->CHECKOUT_TYPE = $config['TRUSTPAY_CHECKOUT_TYPE'];
        }

        if (isset($config['TRUSTPAY_ORDER_ALWAYS'])) {
            $this->ORDER_ALWAYS = $config['TRUSTPAY_ORDER_ALWAYS'];
        }

        if (isset($config['TRUSTPAY_PRIVATE_NOTES'])) {
            $this->PRIVATE_NOTES = $config['TRUSTPAY_PRIVATE_NOTES'];
        }

        /* All statuses (incl. PrestaShop statuses) - Start */
        if (isset($config['TRUSTPAY_STATUS_AWAITING'])) {
            $this->tp_os_awaiting = $config['TRUSTPAY_STATUS_AWAITING'];
        }

        if (isset($config['TRUSTPAY_STATUS_CARDWAITING'])) {
            $this->tp_os_cardwaiting = $config['TRUSTPAY_STATUS_CARDWAITING'];
        }

        if (isset($config['TRUSTPAY_STATUS_PENDING'])) {
            $this->tp_os_pending = $config['TRUSTPAY_STATUS_PENDING'];
        }

        if (isset($config['TRUSTPAY_STATUS_ANNOUNCED'])) {
            $this->tp_os_announced = $config['TRUSTPAY_STATUS_ANNOUNCED'];
        }

        if (isset($config['TRUSTPAY_STATUS_AUTHORIZED'])) {
            $this->tp_os_authorized = $config['TRUSTPAY_STATUS_AUTHORIZED'];
        }

        if (isset($config['TRUSTPAY_STATUS_PROCESSING'])) {
            $this->tp_os_processing = $config['TRUSTPAY_STATUS_PROCESSING'];
        }

        if (isset($config['TRUSTPAY_STATUS_AUTHORIZEDONLY'])) {
            $this->tp_os_authorizedonly = $config['TRUSTPAY_STATUS_AUTHORIZEDONLY'];
        }

        if (isset($config['TRUSTPAY_STATUS_OK'])) {
            $this->STATUS_OK = $config['TRUSTPAY_STATUS_OK'];
        }

        if (isset($config['TRUSTPAY_STATUS_FAIL'])) {
            $this->STATUS_FAIL = $config['TRUSTPAY_STATUS_FAIL'];
        }

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->l('TrustPay');
        $this->description = $this->l('Module for accepting TrustPay (Instant Bank transfers and internal transfers) and TrustCard (Card) payments.').' '.$this->l('Apple Pay is supported.').' '.$this->l('API v.2.0 and SCA comply.');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall TrustPay module ?');

        /* FULL MODULE PATH */
        $this->module_path = Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/';

        /* URLS DEFINITIONS */
        $this->url_wire_live = 'https://ib.trustpay.eu/mapi5/wire/paypopup';
        $this->url_card_live = 'https://ib.trustpay.eu/mapi5/Card/PayPopup';

        $this->url_wire_test = 'https://playground.trustpay.eu/mapi5/wire/paypopup';
        $this->url_card_test = 'https://playground.trustpay.eu/mapi5/Card/PayPopup';

        $this->url_ib_playground = 'https://playground.trustpay.eu/';
        $this->url_ib_live = 'https://ib.trustpay.eu/sk-SK';
        $this->url_faq = 'https://www.trustpay.sk/faq/ib-sk/';

//      $this->js_path = 'https://ib.trustpay.eu/Mapi5/Scripts/TrustPay/popup.js';
//      $this->js_path = 'https://mapi.trustpay.eu/mapi5/Scripts/TrustPay/popup.js';

        /* Return, Cancel & Error URL - Return customer only */
        $this->return_url = Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/response/result.php';
        /* Notification URL - Change order status */
        $this->notif_url = Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/response/notification.php';

        /* IMAGES / ICONS DEFINITIONS */
        $this->logo_trustpay = $this->module_path.'views/img/logo/tp.svg';

        $this->js_path_jq = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/js/jquery.min.js');
        $this->js_popup = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/js/iframe-both.js');
        $this->js_popup_tp = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/js/iframe-wire.js');
        $this->js_popup_tc = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/js/iframe-card.js');

        $this->logo_tp = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/logo/tp.svg');
        $this->logo_tc = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/logo/tc.svg');
        $this->logo_tb = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/logo/tb.svg');
        $this->logo_slsp = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/logo/slsp.svg');
        $this->logo_vub = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/logo/vub.svg');
        $this->logo_pb = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/logo/pb.svg');
        $this->logo_csob = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/logo/csob.svg');
        $this->logo_ib = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/logo/ib.svg');
        $this->logo_visa = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/logo/visa.svg');
        $this->logo_visa_electron = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/logo/visa-electron.svg');
        $this->logo_mastercard = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/logo/mastercard.svg');
        $this->logo_maestro = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/logo/maestro.svg');
        $this->logo_apple_pay = Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/logo/apple-pay.svg');

        /* Warning massages */
        if (!isset($this->AID)) {
            $this->warning = $this->l('Your Merchant account ID must be configured in order to use this module correctly.');
        }

        if (!isset($this->tp_pwd)) {
            $this->warning = $this->l('Your Encryption key must be configured in order to use this module correctly.');
        }

        if (!isset($this->CURR)) {
            $this->warning = $this->l('Account currency must be configured in order to use this module correctly.');
        }

        if (!isset($this->PAYURL)) {
            $this->warning = $this->l('TrustPay domestic bank wire payments status must be configured in order to use this module correctly.');
        }

        if (!isset($this->PAYURLCARD)) {
            $this->warning = $this->l('TrustCard payments status must be configured in order to allow/denied card payments.');
        }

        if (!isset($this->tp_os_awaiting)) {
            $this->warning = $this->l('Payment order status &quot;Awaiting TrustPay payment&quot; is not present.');
        }

        if (!isset($this->tp_os_cardwaiting)) {
            $this->warning = $this->l('Payment order status &quot;Awaiting TrustCard payment&quot; is not present.');
        }

        if (!isset($this->tp_os_pending)) {
            $this->warning = $this->l('Payment order status &quot;Awaiting TrustPay offline payment&quot; is not present.');
        }

        if (!isset($this->tp_os_announced)) {
            $this->warning = $this->l('Payment order status &quot;TrustPay payment announced&quot; is not present.');
        }

        if (!isset($this->tp_os_authorized)) {
            $this->warning = $this->l('Payment order status &quot;TrustPay payment authorized&quot; is not present.');
        }

        if (!isset($this->tp_os_processing)) {
            $this->warning = $this->l('Payment order status &quot;TrustPay payment is processing&quot; is not present.');
        }

        if (!isset($this->tp_os_authorizedonly)) {
            $this->warning = $this->l('Payment order status &quot;Card payment authorized&quot; is not present.');
        }

        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('Warning: No currency has been set for TrustPay module.');
        }
    }

    /******** INSTALL ********/
    public function install()
    {
        if (!parent::install() 
            || !$this->registerHook('payment') 
            || !$this->registerHook('displayPaymentEU') 
            || !$this->registerHook('paymentReturn') 
            || !$this->registerHook('leftColumn') 
            || !$this->registerHook('rightColumn') 
            || !$this->registerHook('displayProductButtons')
            || !$this->registerHook('displayHeader')
            || !$this->registerHook('displayBackOfficeHeader')) {
        return false;
        }

        /* Initial settings */
        Configuration::updateValue('TRUSTPAY_DEBUG', '0');
        Configuration::updateValue('TRUSTPAY_PAYURL', $this->url_wire_test);
        Configuration::updateValue('TRUSTPAY_PAYURLCARD', $this->url_card_test);
        Configuration::updateValue('TRUSTPAY_CUR', 'EUR');
        Configuration::updateValue('TRUSTPAY_LNG', 'sk');
        Configuration::updateValue('TRUSTPAY_CNT', 'SK');
        Configuration::updateValue('TRUSTPAY_DSC', 'Platba z e-shopu');

        /* PrestaShop built-in statuses only */
        Configuration::updateValue('TRUSTPAY_STATUS_OK', Configuration::get('PS_OS_PAYMENT'));
        Configuration::updateValue('TRUSTPAY_STATUS_FAIL', Configuration::get('PS_OS_ERROR'));

        Configuration::updateValue('TRUSTPAY_ORDER_METHOD', 'before');
        Configuration::updateValue('TRUSTPAY_CHECKOUT_TYPE', 'checkout');
        Configuration::updateValue('TRUSTPAY_ORDER_ALWAYS', 'always');
        Configuration::updateValue('TRUSTPAY_THEME', 'thirdparty');
        Configuration::updateValue('TRUSTPAY_ICON', 'p');
        Configuration::updateValue('TRUSTPAY_BANKICON', '1');
        Configuration::updateValue('TRUSTPAY_CARDICON', '1');
        Configuration::updateValue('TRUSTPAY_CMS_ID', '3');

        Configuration::updateValue('TRUSTPAY_PRIVATE_NOTES', 'KONTAKT PRE POMOC: info@trustpay.eu; +421 2 321 68 450');

        /* Copy e-mail templates */
        $this->moveOverriteFiles();

        /* Create a new OrderState */
        if (!Configuration::get('TRUSTPAY_OS_AWAITING')) {
            $orderState = new OrderState();
            $orderState->name = array();
            foreach (Language::getLanguages() as $language) {
                if (strtolower($language['iso_code']) == 'sk') {
                    $orderState->name[$language['id_lang']] = 'Čakanie na platbu TrustPay';
                } elseif (strtolower($language['iso_code']) == 'cs') {
                    $orderState->name[$language['id_lang']] = 'Čekání na platbu TrustPay';
                } elseif (strtolower($language['iso_code']) == 'pl') {
                    $orderState->name[$language['id_lang']] = 'Oczekiwanie na TrustPay płatności';
                } elseif (strtolower($language['iso_code']) == 'hu') {
                    $orderState->name[$language['id_lang']] = 'Várakozás fizetési TrustPay';
                } elseif (strtolower($language['iso_code']) == 'en') {
                    $orderState->name[$language['id_lang']] = 'Awaiting TrustPay payment';
                } else {
                    $orderState->name[$language['id_lang']] = 'Awaiting TrustPay payment';
                }
            }
            $orderState->send_email = false;

            $orderState->template = array();
            foreach (Language::getLanguages() as $language) {
                $orderState->template[$language['id_lang']] = 'trustpay_awaiting';
            }

            $orderState->color = '#4169E1';
            $orderState->hidden = false;
            $orderState->delivery = false;
            $orderState->logable = false;
            $orderState->module_name = $this->name;
            $orderState->invoice = false;
            if ($orderState->add()) {
                copy(dirname(__FILE__).'/views/img/os/'.$this->name.'-awaiting.gif', dirname(__FILE__).'/../../img/os/'.(int)$orderState->id.'.gif');
            }
            Configuration::updateValue('TRUSTPAY_OS_AWAITING', (int)$orderState->id);
        }

        /* Create a new OrderState */
        if (!Configuration::get('TRUSTPAY_OS_CARDWAITING')) {
            $orderState = new OrderState();
            $orderState->name = array();
            foreach (Language::getLanguages() as $language) {
                if (strtolower($language['iso_code']) == 'sk') {
                    $orderState->name[$language['id_lang']] = 'Čakanie na platbu TrustCard';
                } elseif (strtolower($language['iso_code']) == 'cs') {
                    $orderState->name[$language['id_lang']] = 'Čekání na platbu TrustCard';
                } elseif (strtolower($language['iso_code']) == 'pl') {
                    $orderState->name[$language['id_lang']] = 'Oczekiwanie na wypłatę TrustCard';
                } elseif (strtolower($language['iso_code']) == 'hu') {
                    $orderState->name[$language['id_lang']] = 'Vár TrustCard fizetés';    
                } elseif (strtolower($language['iso_code']) == 'en') {
                    $orderState->name[$language['id_lang']] = 'Awaiting TrustCard payment';
                } else {
                    $orderState->name[$language['id_lang']] = 'Awaiting TrustCard payment';
                }
            }
            $orderState->send_email = false;

            $orderState->template = array();
            foreach (Language::getLanguages() as $language) {
                $orderState->template[$language['id_lang']] = 'trustpay_cardwaiting';
            }

            $orderState->color = '#4169E1';
            $orderState->hidden = false;
            $orderState->delivery = false;
            $orderState->logable = false;
            $orderState->module_name = $this->name;
            $orderState->invoice = false;
            if ($orderState->add()) {
                copy(dirname(__FILE__).'/views/img/os/'.$this->name.'-cardwaiting.gif', dirname(__FILE__).'/../../img/os/'.(int)$orderState->id.'.gif');
            }
            Configuration::updateValue('TRUSTPAY_OS_CARDWAITING', (int)$orderState->id);
        }

        /* Create a new OrderState */
        if (!Configuration::get('TRUSTPAY_OS_PENDING')) {
            $orderState = new OrderState();
            $orderState->name = array();
            foreach (Language::getLanguages() as $language) {
                if (strtolower($language['iso_code']) == 'sk') {
                    $orderState->name[$language['id_lang']] = 'Čakanie na offline platbu TrustPay';
                } elseif (strtolower($language['iso_code']) == 'cs') {
                    $orderState->name[$language['id_lang']] = 'Čekání na offline platbu TrustPay';
                } elseif (strtolower($language['iso_code']) == 'pl') {
                    $orderState->name[$language['id_lang']] = 'Oczekiwanie na wypłatę nieaktywny TrustPay';
                } elseif (strtolower($language['iso_code']) == 'hu') {
                    $orderState->name[$language['id_lang']] = 'Vár TrustPay offline fizetési';
                } elseif (strtolower($language['iso_code']) == 'en') {
                    $orderState->name[$language['id_lang']] = 'Awaiting TrustPay offline payment';
                } else {
                    $orderState->name[$language['id_lang']] = 'Awaiting TrustPay offline payment';
                }
            }
            $orderState->send_email = false;

            $orderState->template = array();
            foreach (Language::getLanguages() as $language) {
                $orderState->template[$language['id_lang']] = 'trustpay_pending';
            }

            $orderState->color = '#4169E1';
            $orderState->hidden = false;
            $orderState->delivery = false;
            $orderState->logable = false;
            $orderState->module_name = $this->name;
            $orderState->invoice = false;
            if ($orderState->add()) {
                copy(dirname(__FILE__).'/views/img/os/'.$this->name.'-pending.gif', dirname(__FILE__).'/../../img/os/'.(int)$orderState->id.'.gif');
            }
            Configuration::updateValue('TRUSTPAY_OS_PENDING', (int)$orderState->id);
        }

        /* Create a new OrderState */
        if (!Configuration::get('TRUSTPAY_OS_ANNOUNCED')) {
            $orderState = new OrderState();
            $orderState->name = array();
            foreach (Language::getLanguages() as $language) {
                if (strtolower($language['iso_code']) == 'sk') {
                    $orderState->name[$language['id_lang']] = 'TrustPay platba oznámená';
                } elseif (strtolower($language['iso_code']) == 'cs') {
                    $orderState->name[$language['id_lang']] = 'TrustPay platba oznámena';
                } elseif (strtolower($language['iso_code']) == 'pl') {
                    $orderState->name[$language['id_lang']] = 'Płatność TrustPay ogłosił';
                } elseif (strtolower($language['iso_code']) == 'hu') {
                    $orderState->name[$language['id_lang']] = 'TrustPay fizetési bejelentette';
                } elseif (strtolower($language['iso_code']) == 'en') {
                    $orderState->name[$language['id_lang']] = 'TrustPay payment announced';
                } else {
                    $orderState->name[$language['id_lang']] = 'TrustPay payment announced';
                }
            }
            $orderState->send_email = true;

            $orderState->template = array();
            foreach (Language::getLanguages() as $language) {
                $orderState->template[$language['id_lang']] = 'trustpay_announced';
            }

            $orderState->color = '#BD6211';
            $orderState->hidden = false;
            $orderState->delivery = false;
            $orderState->logable = false;
            $orderState->module_name = $this->name;
            $orderState->invoice = false;
            if ($orderState->add()) {
                copy(dirname(__FILE__).'/views/img/os/'.$this->name.'-announced.gif', dirname(__FILE__).'/../../img/os/'.(int)$orderState->id.'.gif');
            }
            Configuration::updateValue('TRUSTPAY_OS_ANNOUNCED', (int)$orderState->id);
        }

        /* Create a new OrderState */
        if (!Configuration::get('TRUSTPAY_OS_AUTHORIZED')) {
            $orderState = new OrderState();
            $orderState->name = array();
            foreach (Language::getLanguages() as $language) {
                if (strtolower($language['iso_code']) == 'sk') {
                    $orderState->name[$language['id_lang']] = 'TrustPay platba autorizovaná';
                } elseif (strtolower($language['iso_code']) == 'cs') {
                    $orderState->name[$language['id_lang']] = 'TrustPay platba autorizována';
                } elseif (strtolower($language['iso_code']) == 'pl') {
                    $orderState->name[$language['id_lang']] = 'Płatność TrustPay upoważniony';
                } elseif (strtolower($language['iso_code']) == 'hu') {
                    $orderState->name[$language['id_lang']] = 'TrustPay engedélyezett fizetési';
                } elseif (strtolower($language['iso_code']) == 'en') {
                    $orderState->name[$language['id_lang']] = 'TrustPay payment authorized';
                } else {
                    $orderState->name[$language['id_lang']] = 'TrustPay payment authorized';
                }
            }
            $orderState->send_email = true;

            $orderState->template = array();
            foreach (Language::getLanguages() as $language) {
                $orderState->template[$language['id_lang']] = 'trustpay_authorized';
            }

            $orderState->color = '#1B8E6C';
            $orderState->hidden = false;
            $orderState->delivery = false;
            $orderState->logable = false;
            $orderState->module_name = $this->name;
            $orderState->invoice = false;
            if ($orderState->add()) {
                copy(dirname(__FILE__).'/views/img/os/'.$this->name.'-authorized.gif', dirname(__FILE__).'/../../img/os/'.(int)$orderState->id.'.gif');
            }
            Configuration::updateValue('TRUSTPAY_OS_AUTHORIZED', (int)$orderState->id);
        }

        /* Create a new OrderState */
        if (!Configuration::get('TRUSTPAY_OS_PROCESSING')) {
            $orderState = new OrderState();
            $orderState->name = array();
            foreach (Language::getLanguages() as $language) {
                if (strtolower($language['iso_code']) == 'sk') {
                    $orderState->name[$language['id_lang']] = 'TrustPay platba sa spracováva';
                } elseif (strtolower($language['iso_code']) == 'cs') {
                    $orderState->name[$language['id_lang']] = 'TrustPay platba se zpracováva';
                } elseif (strtolower($language['iso_code']) == 'pl') {
                    $orderState->name[$language['id_lang']] = 'Płatność TrustPay przetwarza';
                } elseif (strtolower($language['iso_code']) == 'hu') {
                    $orderState->name[$language['id_lang']] = 'TrustPay fizetés feldolgozás';
                } elseif (strtolower($language['iso_code']) == 'en') {
                    $orderState->name[$language['id_lang']] = 'TrustPay payment is processing';
                } else {
                    $orderState->name[$language['id_lang']] = 'TrustPay payment is processing';
                }
            }
            $orderState->send_email = true;

            $orderState->template = array();
            foreach (Language::getLanguages() as $language) {
                $orderState->template[$language['id_lang']] = 'trustpay_processing';
            }

            $orderState->color = '#1B8E6C';
            $orderState->hidden = false;
            $orderState->delivery = false;
            $orderState->logable = false;
            $orderState->module_name = $this->name;
            $orderState->invoice = false;
            if ($orderState->add()) {
                copy(dirname(__FILE__).'/views/img/os/'.$this->name.'-processing.gif', dirname(__FILE__).'/../../img/os/'.(int)$orderState->id.'.gif');
            }
            Configuration::updateValue('TRUSTPAY_OS_PROCESSING', (int)$orderState->id);
        }

        /* Create a new OrderState */
        if (!Configuration::get('TRUSTPAY_OS_AUTHORIZEDONLY')) {
            $orderState = new OrderState();
            $orderState->name = array();
            foreach (Language::getLanguages() as $language) {
                if (strtolower($language['iso_code']) == 'sk') {
                    $orderState->name[$language['id_lang']] = 'Platba kartou autorizovaná';
                } elseif (strtolower($language['iso_code']) == 'cs') {
                    $orderState->name[$language['id_lang']] = 'Platba kartou autorizována';
                } elseif (strtolower($language['iso_code']) == 'pl') {
                    $orderState->name[$language['id_lang']] = 'Płatność kartą upoważniony';
                } elseif (strtolower($language['iso_code']) == 'hu') {
                    $orderState->name[$language['id_lang']] = 'Bankkártyás fizetés engedélyezett';
                } elseif (strtolower($language['iso_code']) == 'en') {
                    $orderState->name[$language['id_lang']] = 'Card payment authorized';
                } else {
                    $orderState->name[$language['id_lang']] = 'Card payment authorized';
                }
            }
            $orderState->send_email = true;

            $orderState->template = array();
            foreach (Language::getLanguages() as $language) {
                $orderState->template[$language['id_lang']] = 'trustpay_authorizedonly';
            }

            $orderState->color = '#BD6211';
            $orderState->hidden = false;
            $orderState->delivery = false;
            $orderState->logable = false;
            $orderState->module_name = $this->name;
            $orderState->invoice = false;
            if ($orderState->add()) {
                copy(dirname(__FILE__).'/views/img/os/'.$this->name.'-authorizedonly.gif', dirname(__FILE__).'/../../img/os/'.(int)$orderState->id.'.gif');
            }
            Configuration::updateValue('TRUSTPAY_OS_AUTHORIZEDONLY', (int)$orderState->id);
        }

        /* Module statuses only - Start */
        Configuration::updateValue('TRUSTPAY_STATUS_AWAITING', Configuration::get('TRUSTPAY_OS_AWAITING'));
        Configuration::updateValue('TRUSTPAY_STATUS_CARDWAITING', Configuration::get('TRUSTPAY_OS_CARDWAITING'));
        Configuration::updateValue('TRUSTPAY_STATUS_PENDING', Configuration::get('TRUSTPAY_OS_PENDING'));
        Configuration::updateValue('TRUSTPAY_STATUS_ANNOUNCED', Configuration::get('TRUSTPAY_OS_ANNOUNCED'));
        Configuration::updateValue('TRUSTPAY_STATUS_AUTHORIZED', Configuration::get('TRUSTPAY_OS_AUTHORIZED'));
        Configuration::updateValue('TRUSTPAY_STATUS_PROCESSING', Configuration::get('TRUSTPAY_OS_PROCESSING'));
        Configuration::updateValue('TRUSTPAY_STATUS_AUTHORIZEDONLY', Configuration::get('TRUSTPAY_OS_AUTHORIZEDONLY'));
        /* Module statuses only - End */

        $whitelist = array('127.0.0.1', '::1');
        if (!in_array($_SERVER['REMOTE_ADDR'], $whitelist)) {
            $this->_notifyMe();
        }

        return true;
    }

    /******** UNINSTALL ********/
    public function uninstall()
    {
        if (!$this->unregisterHook('payment') 
                || !$this->unregisterHook('displayPaymentEU')
                || !$this->unregisterHook('paymentReturn')
                || !$this->unregisterHook('leftColumn')
                || !$this->unregisterHook('rightColumn')
                || !$this->unregisterHook('displayProductButtons')
                || !$this->unregisterHook('displayHeader')
                || !$this->unregisterHook('displayBackOfficeHeader')
                || !Configuration::deleteByName('TRUSTPAY_DEBUG')
                || !Configuration::deleteByName('TRUSTPAY_AID')
                || !Configuration::deleteByName('TRUSTPAY_PAYURL')
                || !Configuration::deleteByName('TRUSTPAY_PAYURLCARD')
                || !Configuration::deleteByName('TRUSTPAY_CUR')
                || !Configuration::deleteByName('TRUSTPAY_CNT')
                || !Configuration::deleteByName('TRUSTPAY_LNG')
                || !Configuration::deleteByName('TRUSTPAY_DSC')
                || !Configuration::deleteByName('TRUSTPAY_PWD')

                || !Configuration::deleteByName('TRUSTPAY_ORDER_METHOD')
                || !Configuration::deleteByName('TRUSTPAY_CHECKOUT_TYPE')
                || !Configuration::deleteByName('TRUSTPAY_ORDER_ALWAYS')
                || !Configuration::deleteByName('TRUSTPAY_STATUS_AWAITING') 
                || !Configuration::deleteByName('TRUSTPAY_STATUS_CARDWAITING') 
                || !Configuration::deleteByName('TRUSTPAY_STATUS_PENDING') 
                || !Configuration::deleteByName('TRUSTPAY_STATUS_ANNOUNCED') 
                || !Configuration::deleteByName('TRUSTPAY_STATUS_AUTHORIZED') 
                || !Configuration::deleteByName('TRUSTPAY_STATUS_PROCESSING') 
                || !Configuration::deleteByName('TRUSTPAY_STATUS_AUTHORIZEDONLY') 
                || !Configuration::deleteByName('TRUSTPAY_STATUS_OK') 
                || !Configuration::deleteByName('TRUSTPAY_STATUS_FAIL') 

                || !Configuration::deleteByName('TRUSTPAY_THEME')
                || !Configuration::deleteByName('TRUSTPAY_ICON')
                || !Configuration::deleteByName('TRUSTPAY_BANKICON')
                || !Configuration::deleteByName('TRUSTPAY_CARDICON')
                || !Configuration::deleteByName('TRUSTPAY_CMS_ID')

                || !Configuration::deleteByName('TRUSTPAY_PRIVATE_NOTES')
/*
                || !Configuration::deleteByName('TRUSTPAY_OS_AWAITING') 
                || !Configuration::deleteByName('TRUSTPAY_OS_CARDWAITING') 
                || !Configuration::deleteByName('TRUSTPAY_OS_PENDING') 
                || !Configuration::deleteByName('TRUSTPAY_OS_ANNOUNCED') 
                || !Configuration::deleteByName('TRUSTPAY_OS_AUTHORIZED') 
                || !Configuration::deleteByName('TRUSTPAY_OS_PROCESSING') 
                || !Configuration::deleteByName('TRUSTPAY_OS_AUTHORIZEDONLY')
*/ 
                || !parent::uninstall()) {
            return false;
        }
        return true;
    }

    /******** COPY E-MAIL TEMPLATES INTO /mails/ FOLDER ********/
    private function moveOverriteFiles()
    {
         foreach (Language::getLanguages() as $language) {
             if (strtolower($language['iso_code']) == 'sk') {

                if (!copy(dirname(__FILE__).'/mails/sk/'.$this->name.'_announced.html',_PS_ROOT_DIR_ .'/mails/sk/'.$this->name.'_announced.html') 
                        || !copy(dirname(__FILE__).'/mails/sk/'.$this->name.'_announced.txt',_PS_ROOT_DIR_ .'/mails/sk/'.$this->name.'_announced.txt') 
                        || !copy(dirname(__FILE__).'/mails/sk/'.$this->name.'_authorized.html',_PS_ROOT_DIR_ .'/mails/sk/'.$this->name.'_authorized.html')
                        || !copy(dirname(__FILE__).'/mails/sk/'.$this->name.'_authorized.txt',_PS_ROOT_DIR_ .'/mails/sk/'.$this->name.'_authorized.txt')
                        || !copy(dirname(__FILE__).'/mails/sk/'.$this->name.'_pending.html',_PS_ROOT_DIR_ .'/mails/sk/'.$this->name.'_pending.html')
                        || !copy(dirname(__FILE__).'/mails/sk/'.$this->name.'_pending.txt',_PS_ROOT_DIR_ .'/mails/sk/'.$this->name.'_pending.txt')
                        || !copy(dirname(__FILE__).'/mails/sk/'.$this->name.'_processing.html',_PS_ROOT_DIR_ .'/mails/sk/'.$this->name.'_processing.html')
                        || !copy(dirname(__FILE__).'/mails/sk/'.$this->name.'_processing.txt',_PS_ROOT_DIR_ .'/mails/sk/'.$this->name.'_processing.txt')
                        || !copy(dirname(__FILE__).'/mails/sk/'.$this->name.'_authorizedonly.html',_PS_ROOT_DIR_ .'/mails/sk/'.$this->name.'_authorizedonly.html')
                        || !copy(dirname(__FILE__).'/mails/sk/'.$this->name.'_authorizedonly.txt',_PS_ROOT_DIR_ .'/mails/sk/'.$this->name.'_authorizedonly.txt')
                        || !copy(dirname(__FILE__).'/mails/sk/'.$this->name.'_cardwaiting.html',_PS_ROOT_DIR_ .'/mails/sk/'.$this->name.'_cardwaiting.html')
                        || !copy(dirname(__FILE__).'/mails/sk/'.$this->name.'_cardwaiting.txt',_PS_ROOT_DIR_ .'/mails/sk/'.$this->name.'_cardwaiting.txt')
                        || !copy(dirname(__FILE__).'/mails/sk/'.$this->name.'_awaiting.html',_PS_ROOT_DIR_ .'/mails/sk/'.$this->name.'_awaiting.html')
                        || !copy(dirname(__FILE__).'/mails/sk/'.$this->name.'_awaiting.txt',_PS_ROOT_DIR_ .'/mails/sk/'.$this->name.'_awaiting.txt')) {
                    return false;
                }

             } elseif (strtolower($language['iso_code']) == 'cs') {

                if (!copy(dirname(__FILE__).'/mails/cs/'.$this->name.'_announced.html',_PS_ROOT_DIR_ .'/mails/cs/'.$this->name.'_announced.html') 
                        || !copy(dirname(__FILE__).'/mails/cs/'.$this->name.'_announced.txt',_PS_ROOT_DIR_ .'/mails/cs/'.$this->name.'_announced.txt') 
                        || !copy(dirname(__FILE__).'/mails/cs/'.$this->name.'_authorized.html',_PS_ROOT_DIR_ .'/mails/cs/'.$this->name.'_authorized.html')
                        || !copy(dirname(__FILE__).'/mails/cs/'.$this->name.'_authorized.txt',_PS_ROOT_DIR_ .'/mails/cs/'.$this->name.'_authorized.txt')
                        || !copy(dirname(__FILE__).'/mails/cs/'.$this->name.'_pending.html',_PS_ROOT_DIR_ .'/mails/cs/'.$this->name.'_pending.html')
                        || !copy(dirname(__FILE__).'/mails/cs/'.$this->name.'_pending.txt',_PS_ROOT_DIR_ .'/mails/cs/'.$this->name.'_pending.txt')
                        || !copy(dirname(__FILE__).'/mails/cs/'.$this->name.'_processing.html',_PS_ROOT_DIR_ .'/mails/cs/'.$this->name.'_processing.html')
                        || !copy(dirname(__FILE__).'/mails/cs/'.$this->name.'_processing.txt',_PS_ROOT_DIR_ .'/mails/cs/'.$this->name.'_processing.txt')
                        || !copy(dirname(__FILE__).'/mails/cs/'.$this->name.'_authorizedonly.html',_PS_ROOT_DIR_ .'/mails/cs/'.$this->name.'_authorizedonly.html')
                        || !copy(dirname(__FILE__).'/mails/cs/'.$this->name.'_authorizedonly.txt',_PS_ROOT_DIR_ .'/mails/cs/'.$this->name.'_authorizedonly.txt')
                        || !copy(dirname(__FILE__).'/mails/cs/'.$this->name.'_cardwaiting.html',_PS_ROOT_DIR_ .'/mails/cs/'.$this->name.'_cardwaiting.html')
                        || !copy(dirname(__FILE__).'/mails/cs/'.$this->name.'_cardwaiting.txt',_PS_ROOT_DIR_ .'/mails/cs/'.$this->name.'_cardwaiting.txt')
                        || !copy(dirname(__FILE__).'/mails/cs/'.$this->name.'_awaiting.html',_PS_ROOT_DIR_ .'/mails/cs/'.$this->name.'_awaiting.html')
                        || !copy(dirname(__FILE__).'/mails/cs/'.$this->name.'_awaiting.txt',_PS_ROOT_DIR_ .'/mails/cs/'.$this->name.'_awaiting.txt')) {
                    return false;
                }

             } elseif (strtolower($language['iso_code']) == 'en') {

                if (!copy(dirname(__FILE__).'/mails/en/'.$this->name.'_announced.html',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_announced.html') 
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_announced.txt',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_announced.txt') 
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_authorized.html',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_authorized.html')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_authorized.txt',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_authorized.txt')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_pending.html',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_pending.html')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_pending.txt',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_pending.txt')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_processing.html',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_processing.html')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_processing.txt',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_processing.txt')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_authorizedonly.html',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_authorizedonly.html')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_authorizedonly.txt',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_authorizedonly.txt')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_cardwaiting.html',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_cardwaiting.html')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_cardwaiting.txt',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_cardwaiting.txt')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_awaiting.html',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_awaiting.html')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_awaiting.txt',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_awaiting.txt')) {
                    return false;
                }

             } else {

                if (!copy(dirname(__FILE__).'/mails/en/'.$this->name.'_announced.html',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_announced.html') 
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_announced.txt',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_announced.txt') 
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_authorized.html',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_authorized.html')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_authorized.txt',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_authorized.txt')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_pending.html',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_pending.html')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_pending.txt',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_pending.txt')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_processing.html',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_processing.html')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_processing.txt',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_processing.txt')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_authorizedonly.html',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_authorizedonly.html')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_authorizedonly.txt',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_authorizedonly.txt')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_cardwaiting.html',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_cardwaiting.html')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_cardwaiting.txt',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_cardwaiting.txt')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_awaiting.html',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_awaiting.html')
                        || !copy(dirname(__FILE__).'/mails/en/'.$this->name.'_awaiting.txt',_PS_ROOT_DIR_ .'/mails/en/'.$this->name.'_awaiting.txt')) {
                    return false;
                }

             }
         }

        return true;
    }

    /******** POST VALIDATION ********/
    private function _postValidation()
    {
        if (Tools::isSubmit('btnSubmit')) {
            if (!Tools::getValue('TRUSTPAY_AID')) {
                $this->_postErrors[] = $this->l('Merchant account ID parameter is required.');
            } elseif (!Tools::getValue('TRUSTPAY_PWD')) {
                $this->_postErrors[] = $this->l('Encryption key is required.');
            } elseif (!Tools::getValue('TRUSTPAY_CUR')) {
                $this->_postErrors[] = $this->l('Account currency type parameter is required.');
            }
        } elseif (Tools::isSubmit('btnSubmitOrder')) {
            if (!Tools::getValue('TRUSTPAY_CHECKOUT_TYPE')) {
                $this->_postErrors[] = $this->l('Choose one of the checkout type.');
            } elseif (!Tools::getValue('TRUSTPAY_ORDER_ALWAYS')) {
                $this->_postErrors[] = $this->l('Choose one of the order creation option.');
            } elseif (!Tools::getValue('TRUSTPAY_ORDER_METHOD')) {
                $this->_postErrors[] = $this->l('Choose one of the order method.');
            } elseif (!Tools::getValue('TRUSTPAY_STATUS_AWAITING')) {
                $this->_postErrors[] = $this->l('Awaiting payment order status is required.');
            } elseif (!Tools::getValue('TRUSTPAY_STATUS_CARDWAITING')) {
                $this->_postErrors[] = $this->l('Awaiting card payment order status is required.');
            } elseif (!Tools::getValue('TRUSTPAY_STATUS_PENDING')) {
                $this->_postErrors[] = $this->l('Awaiting offline payment order status is required.');
            } elseif (!Tools::getValue('TRUSTPAY_STATUS_ANNOUNCED')) {
                $this->_postErrors[] = $this->l('Announced payment order status is required.');
            } elseif (!Tools::getValue('TRUSTPAY_STATUS_AUTHORIZED')) {
                $this->_postErrors[] = $this->l('Authorized payment order status is required.');
            } elseif (!Tools::getValue('TRUSTPAY_STATUS_PROCESSING')) {
                $this->_postErrors[] = $this->l('Processing payment order status is required.');
            } elseif (!Tools::getValue('TRUSTPAY_STATUS_AUTHORIZEDONLY')) {
                $this->_postErrors[] = $this->l('Card authorized payment order status is required.');
            } elseif (!Tools::getValue('TRUSTPAY_STATUS_OK')) {
                $this->_postErrors[] = $this->l('Success order status is required.');
            } elseif (!Tools::getValue('TRUSTPAY_STATUS_FAIL')) {
                $this->_postErrors[] = $this->l('Error order status is required.');
            }
        }
    }

    /******** POST PROCESS ********/
    private function _postProcess()
    {
        if (Tools::isSubmit('btnSubmit')) {
            Configuration::updateValue('TRUSTPAY_DEBUG', Tools::getValue('TRUSTPAY_DEBUG'));
            Configuration::updateValue('TRUSTPAY_AID', Tools::getValue('TRUSTPAY_AID'));
            Configuration::updateValue('TRUSTPAY_PWD', Tools::getValue('TRUSTPAY_PWD'), true);
            Configuration::updateValue('TRUSTPAY_CUR', Tools::getValue('TRUSTPAY_CUR'));
            Configuration::updateValue('TRUSTPAY_LNG', Tools::getValue('TRUSTPAY_LNG'));
            Configuration::updateValue('TRUSTPAY_CNT', Tools::getValue('TRUSTPAY_CNT'));
            Configuration::updateValue('TRUSTPAY_DSC', Tools::getValue('TRUSTPAY_DSC'));
            Configuration::updateValue('TRUSTPAY_PAYURL', Tools::getValue('TRUSTPAY_PAYURL'));
            Configuration::updateValue('TRUSTPAY_PAYURLCARD', Tools::getValue('TRUSTPAY_PAYURLCARD'));
          $this->_html .= $this->displayConfirmation($this->l('Main module settings updated successfuly'));
        } elseif (Tools::isSubmit('btnSubmitOrder')) {
            Configuration::updateValue('TRUSTPAY_ORDER_METHOD', Tools::getValue('TRUSTPAY_ORDER_METHOD'));
            Configuration::updateValue('TRUSTPAY_CHECKOUT_TYPE', Tools::getValue('TRUSTPAY_CHECKOUT_TYPE'));
            Configuration::updateValue('TRUSTPAY_ORDER_ALWAYS', Tools::getValue('TRUSTPAY_ORDER_ALWAYS'));
            Configuration::updateValue('TRUSTPAY_STATUS_AWAITING', Tools::getValue('TRUSTPAY_STATUS_AWAITING'));
            Configuration::updateValue('TRUSTPAY_STATUS_CARDWAITING', Tools::getValue('TRUSTPAY_STATUS_CARDWAITING'));
            Configuration::updateValue('TRUSTPAY_STATUS_PENDING', Tools::getValue('TRUSTPAY_STATUS_PENDING'));
            Configuration::updateValue('TRUSTPAY_STATUS_ANNOUNCED', Tools::getValue('TRUSTPAY_STATUS_ANNOUNCED'));
            Configuration::updateValue('TRUSTPAY_STATUS_AUTHORIZED', Tools::getValue('TRUSTPAY_STATUS_AUTHORIZED'));
            Configuration::updateValue('TRUSTPAY_STATUS_PROCESSING', Tools::getValue('TRUSTPAY_STATUS_PROCESSING'));
            Configuration::updateValue('TRUSTPAY_STATUS_AUTHORIZEDONLY', Tools::getValue('TRUSTPAY_STATUS_AUTHORIZEDONLY'));
            Configuration::updateValue('TRUSTPAY_STATUS_OK', Tools::getValue('TRUSTPAY_STATUS_OK'));
            Configuration::updateValue('TRUSTPAY_STATUS_FAIL', Tools::getValue('TRUSTPAY_STATUS_FAIL'));
            $this->_html .= $this->displayConfirmation($this->l('The order settings updated successfuly'));
        } elseif (Tools::isSubmit('btnSubmitIcon')) {
            Configuration::updateValue('TRUSTPAY_THEME', Tools::getValue('TRUSTPAY_THEME'));
            Configuration::updateValue('TRUSTPAY_CARDICON', Tools::getValue('TRUSTPAY_CARDICON'));
            Configuration::updateValue('TRUSTPAY_ICON', Tools::getValue('TRUSTPAY_ICON'));
            Configuration::updateValue('TRUSTPAY_BANKICON', Tools::getValue('TRUSTPAY_BANKICON'));
            Configuration::updateValue('TRUSTPAY_CMS_ID', Tools::getValue('TRUSTPAY_CMS_ID'));
            $this->_html .= $this->displayConfirmation($this->l('Design settings updated successfuly'));
        } elseif (Tools::isSubmit('btnSubmitPrivate')) {
            Configuration::updateValue('TRUSTPAY_PRIVATE_NOTES', Tools::getValue('TRUSTPAY_PRIVATE_NOTES'), true);
            $this->_html .= $this->displayConfirmation($this->l('Private notes updated successfuly'));
        }
    }

    /******** MODULE INFOS ********/
    private function _displayInfos()
    {
        $this->smarty->assign(array(
            'module_name' => $this->displayName,
            'module_filename' => $this->name,
            'module_desc' => $this->description,
            'module_version' => $this->version.' - '.$this->rdate,
            'url_ib_playground' => $this->url_ib_playground,
            'url_ib_live' => $this->url_ib_live,
            'url_faq' => $this->url_faq,
            'logo_image_alt' => $this->l('Logo:').' '.$this->l('TrustPay gateway'),
            'text_module_version' => $this->l('version'),
            'text_module_purpose' => $this->l('TrustPay Inc. is a member of Meracrest group and approved by National Bank of Slovakia.'),
            'text_btn_playground' => $this->l('TrustPay Playground'),
            'text_btn_live' => $this->l('TrustPay Internet Banking'),
            'text_btn_faq' => $this->l('FAQ'),
            'module_logo' => $this->logo_trustpay,
            'this_path' => $this->_path,
            'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/',
        ));

        return $this->display(__FILE__, 'mod_infos.tpl');
    }
    
    /******** MODULE CREDIT ********/
    private function _displayCredit()
    {
        $this->smarty->assign(array(
            'module_name' => $this->displayName,
            'module_filename' => $this->name,
            'module_changelog' => Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/CHANGELOG.txt'),
            'module_readme' => Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/docs/readme-'.$this->name.'.php'),
            'module_license' => Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/docs/terms-and-conditions-of-use.pdf'),
            'url_readme_anchor' => $this->l('Module documentation'),
            'url_changelog_anchor' => $this->l('Module Changelog'),
            'url_license_anchor' => $this->l('End-user license agreement'),
            'url_author_anchor' => $this->l('Module author:'),
            'url_help_anchor' => $this->l('PrestaShop help and support:'),
            'url_marketplace_anchor' => $this->l('PrestaShop Modules & Themes Marketplace:'),
            'this_path' => $this->_path,
            'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/',
        ));

        return $this->display(__FILE__, 'mod_credit.tpl');
    }

    /******** GET CONTENT ********/
    public function getContent()
    {
        $this->_html = '';

        $this->_html .= $this->_displayInfos();
        $this->_html .= $this->_displayCredit();

        if (Tools::isSubmit('btnSubmit') || Tools::isSubmit('btnSubmitOrder') || Tools::isSubmit('btnSubmitIcon') || Tools::isSubmit('btnSubmitPrivate')) {
            $this->_postValidation();
            if (!count($this->_postErrors)) {
                $this->_postProcess();
            } else {
                foreach ($this->_postErrors as $err) {
                    $this->_html .= $this->displayError($err);
                }
            }
        } else {
            $this->_html .= '';
        }

        $this->_html .= $this->renderFormGeneral();
        $this->_html .= $this->renderFormOrder();
        $this->_html .= $this->renderFormIcon();
        $this->_html .= $this->renderFormPrivatenotes();
        $this->_html .= $this->_displayOffer();

        return $this->_html;
    }

    /******** CREATE SIGNATURE *******/
    public function GetSignature($key, $message)
    {
        $signature = '';

        if (function_exists('hash_hmac')) {
            $signature = strtoupper(hash_hmac('sha256', pack('A*', $message), pack('A*', $key)));
        } else {
            $this->_postErrors[] = $this->l($this->displayName.': Function hash_hmac doesn\'t exist on your server. Contact your webhosting provider.');
        }
          
        return $signature;
    }

    /******** CHECKING IF AN ORDER EXIST *******/
    public function _getOrderIdExist($cart_id)
    {
        return (int)Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue('SELECT `id_order` FROM `'._DB_PREFIX_.'orders` WHERE `id_cart` = '.$cart_id.'');
    }

    /******** GET ORDER REFERENCE BY ORDER ID *******/
    public function _getReferenceByOrderId($order_id)
    {
        return Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue('SELECT `reference` FROM `'._DB_PREFIX_.'orders` WHERE `id_order` = '.$order_id.'');
    }

    /******** CURRENCY CHECK RESTRICTIONS ********/
    private function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    /******** PAYMENT OPTIONS ********/
    public function hookPayment($params)
    {
        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
//          return;
        }

        $cart = $this->context->cart;

        /* Customer's invoice address details */
        $invoice_address = new Address($cart->id_address_invoice);
        $invoice_address_country = new Country($invoice_address->id_country);

        $cardholder = $invoice_address->firstname.' '.$invoice_address->lastname;
        $billingstreet = $invoice_address->address1;
        if (isset($invoice_address->address2)) {
            $billingstreet .= ' '.$invoice_address->address2;
        }
        $billingcity = $invoice_address->city;
        $billingpostcode = str_replace(' ', '', $invoice_address->postcode);
        $billingcountry = strtoupper($invoice_address_country->iso_code);
        $email = $this->context->cookie->email;

        $total = number_format($cart->getOrderTotal(true, 3), 2, '.', '');
        $paymenttype = '0';

        /* Create signature - Wire payments */
        $message = sprintf("%d/%s/%s/%s/%d", $this->AID, $total, $this->context->currency->iso_code, $cart->id, $paymenttype);
        $signature = $this->GetSignature(Configuration::get('TRUSTPAY_PWD'), $message);

        /* Create signature - Card payments */
        $message_card = sprintf("%d/%s/%s/%s/%d/%s/%s/%s/%s/%s/%s", $this->AID, $total, $this->context->currency->iso_code, $cart->id, $paymenttype, $billingcity, $billingcountry, $billingpostcode, $billingstreet, $cardholder, $email);
        $signature_card = $this->GetSignature(Configuration::get('TRUSTPAY_PWD'), $message_card);

        /* Building payment URL */
        /* DOCS: https://doc.trustpay.eu/#wire-payments */
        $url_tp = $this->PAYURL.'?';
        $url_tp .= 'AccountId='.$this->AID;
        $url_tp .= '&Amount='.$total;
        $url_tp .= '&Currency='.$this->context->currency->iso_code;
        $url_tp .= '&Reference='.$cart->id;
        $url_tp .= '&PaymentType='.$paymenttype;
        $url_tp .= '&Signature='.$signature;
        $url_tp .= '&ReturnUrl='.$this->return_url;
        $url_tp .= '&CancelUrl='.$this->return_url;
        $url_tp .= '&ErrorUrl='.$this->return_url;
        $url_tp .= '&NotificationUrl='.$this->notif_url;
        if (isset($this->LANGUAGE)) {
            $url_tp .= '&Localization='.$this->LANGUAGE;
        }
        if (isset($this->COUNTRY)) {
            $url_tp .= '&Country='.$this->COUNTRY;
        }
        if (isset($this->DESCRIPTION)) {
            $url_tp .= '&Description='.urlencode($this->DESCRIPTION);
        } else {
            $url_tp .= '&Description='.urlencode('ID kosika: ').$cart->id;
        }
        if (isset($this->context->cookie->email)) {
            $url_tp .= '&Email='.urlencode($this->context->cookie->email);
        }

        /* DOCS: https://doc.trustpay.eu/#cards-purchase */
        $url_tc = $this->PAYURLCARD.'?';
        $url_tc .= 'AccountId='.$this->AID;
        $url_tc .= '&Amount='.$total;
        $url_tc .= '&Currency='.$this->context->currency->iso_code;
        $url_tc .= '&Reference='.$cart->id;
        $url_tc .= '&PaymentType='.$paymenttype;
        $url_tc .= '&Signature='.$signature_card;
        $url_tc .= '&BillingCity='.$billingcity;
        $url_tc .= '&BillingCountry='.$billingcountry;
        $url_tc .= '&BillingPostcode='.$billingpostcode;
        $url_tc .= '&BillingStreet='.$billingstreet;
        $url_tc .= '&CardHolder='.$cardholder;
        $url_tc .= '&Email='.urlencode($email);
        $url_tc .= '&ReturnUrl='.$this->return_url;
        $url_tc .= '&CancelUrl='.$this->return_url;
        $url_tc .= '&ErrorUrl='.$this->return_url;
        $url_tc .= '&NotificationUrl='.$this->notif_url;
        if (isset($this->LANGUAGE)) {
            $url_tc .= '&Localization='.$this->LANGUAGE;
        }
        if (isset($this->COUNTRY)) {
            $url_tc .= '&Country='.$this->COUNTRY;
        }
        if (isset($this->DESCRIPTION)) {
            $url_tc .= '&Description='.urlencode($this->DESCRIPTION);
        } else {
            $url_tc .= '&Description='.urlencode('ID kosika: ').$cart->id;
        }

        $this->smarty->assign(array(
            'module_name' => $this->displayName,
            'module_filename' => $this->name,
            'checkout_type' => $this->CHECKOUT_TYPE,
            'js_path_jq' => $this->js_path_jq,
            'js_popup_both' => $this->js_popup,
            'js_popup_tp' => $this->js_popup_tp,
            'js_popup_tc' => $this->js_popup_tc,
            'pay_url_tp' => $url_tp,
            'pay_url_tc' => $url_tc,

            'aid' => $this->AID,
            'pwd' => $this->tp_pwd,

            'bankwirepayment_status' => $this->PAYURL,
            'trustcard_status' => $this->PAYURLCARD,

            'payment_info_tp' => $this->l('Pay in your domestic bank by TrustPay gateway or by TrustPay internal transfers.'),
            'payment_requirements' => $this->l('Please, prepare following card information: Card number, Expiration date and CVV number.'),
            'img_alt_tp' => $this->l('Logo:').' '.$this->l('TrustPay'),
            'img_alt_banks' => $this->l('Logo:').' '.$this->l('Domestic banks'),

            'payment_info_tc' => $this->l('Pay by your VISA, VISA Electron, MasterCard or Maestro payment card.').' '.$this->l('Apple Pay is supported.'),
            'img_alt_tc' => $this->l('Logo:').' '.$this->l('TrustCard'),
            'img_alt_visa' => $this->l('Logo:').' '.$this->l('VISA'),
            'img_alt_visa_electron' => $this->l('Logo:').' '.$this->l('VISA Electron'),
            'img_alt_mastercard' => $this->l('Logo:').' '.$this->l('MasterCard'),
            'img_alt_maestro' => $this->l('Logo:').' '.$this->l('Maestro'),
            'img_alt_apple_pay' => $this->l('Logo:').' '.$this->l('Apple Pay'),
            'img_alt_cards' => $this->l('Logos:').' '.$this->l('VISA, VISA Electron, MasterCard & Maestro'),

            'logo_trustpay' => Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/'.$this->name.'-icon.png'),
            'logo_trustcard' => Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/trustcard-icon.png'),
            'show_icon_banks' => $this->BANKICON,
            'show_icon_cards' => $this->CARDICON,

            'logo_tp' => $this->logo_tp,
            'logo_tc' => $this->logo_tc,
            'logo_tb' => $this->logo_tb,
            'logo_slsp' => $this->logo_slsp,
            'logo_vub' => $this->logo_vub,
            'logo_pb' => $this->logo_pb,
            'logo_csob' => $this->logo_csob,
            'logo_ib' => $this->logo_ib,
            'logo_visa' => $this->logo_visa,
            'logo_visa_electron' => $this->logo_visa_electron,
            'logo_mastercard' => $this->logo_mastercard,
            'logo_maestro' => $this->logo_maestro,
            'logo_apple_pay' => $this->logo_apple_pay,

            'icon_cards' => Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/icons-cards.png'),

            'this_path' => $this->_path,
            'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/',
            ));

        return $this->display(__FILE__, 'payment_intro.tpl');
    }

    /******** PAYMENT EU ********/
    public function hookDisplayPaymentEU($params)
    {
        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
//          return;
        }

        $payment_options = array();

        if ($this->PAYURL != 'disable') {
            $po_trustpay = array(
                'cta_text' => $this->l('TrustPay').'&nbsp;&nbsp;<small>'.$this->l('(Online bank and internal transfers)').'</small>',
                'logo' => Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/trustpay-icon.png'),
                'action' => $this->context->link->getModuleLink($this->name, 'payment', array(), true).'?type=trustpay'
            );
        } else {
            $po_trustpay = '0';
        }

        if ($this->PAYURLCARD != 'disable') {
            $po_trustcard = array(
                'cta_text' => $this->l('TrustCard').'&nbsp;&nbsp;<small>'.$this->l('(Card payment)').'</small>',
                'logo' => Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/trustcard-icon.png'),
                'action' => $this->context->link->getModuleLink($this->name, 'payment', array(), true).'?type=trustcard'
            );
        } else {
            $po_trustcard = 0;
        }

        if (is_array($po_trustpay)) {
            array_push($payment_options, $po_trustpay);
        }

        if (is_array($po_trustcard)) {
            array_push($payment_options, $po_trustcard);
        }

        return $payment_options;
    }

    /******** PAYMENT RETURN ********/
    public function hookPaymentReturn($params)
    {
        if (!$this->active) {
            return;
        }

        $state = $params['objOrder']->getCurrentState();

            $this->smarty->assign(array(
                'module_name' => $this->displayName,
                'module_filename' => $this->name,
                'id_order' => $params['objOrder']->id,
                'reference' => $params['objOrder']->reference,
                'contact_url' => $this->context->link->getPageLink('contact', true),
                'history_url' => $this->context->link->getPageLink('history', true),
                'order_url' => $this->context->link->getPageLink('order', null, null, 'step=3'),
                'total' => Tools::displayPrice($params['total_to_pay'], $params['currencyObj'], false),
                'this_path' => $this->_path,
                'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/',
            ));

        if (in_array($state, array(Configuration::get('TRUSTPAY_OS_AWAITING'),
                            Configuration::get('TRUSTPAY_OS_CARDWAITING')))) {

            $this->smarty->assign(array('status' => 'authorized'));

        } elseif (in_array($state, array(
                            Configuration::get('TRUSTPAY_STATUS_OK'),
                            Configuration::get('PS_OS_OUTOFSTOCK'),
                            Configuration::get('PS_OS_OUTOFSTOCK_UNPAID')))) {

            $this->smarty->assign(array('status' => 'ok'));

        } elseif (in_array($state, array(Configuration::get('TRUSTPAY_OS_PENDING')))) {

            $this->smarty->assign(array('status' => 'pending'));

        } elseif (in_array($state, array(Configuration::get('TRUSTPAY_OS_ANNOUNCED')))) {

            $this->smarty->assign(array('status' => 'announced'));

        } elseif (in_array($state, array(Configuration::get('TRUSTPAY_OS_AUTHORIZED'),
                            Configuration::get('TRUSTPAY_OS_PROCESSING'),
                            Configuration::get('TRUSTPAY_OS_AUTHORIZEDONLY')))) {

            $this->smarty->assign(array('status' => 'authorized'));

        } elseif (in_array($state, array(
                            Configuration::get('TRUSTPAY_STATUS_FAIL'),
                            Configuration::get('PS_OS_ERROR')))) {

            $this->smarty->assign(array('status' => 'failed'));

        } else {

            $this->smarty->assign(array('status' => 'error'));
        }

        return $this->display(__FILE__, 'payment_return.tpl');
    }

    /******** CREATE AND ADD PRIVATE MESSAGE TO AN ORDER ********/
    public function _addNewPrivateMessage($id_order, $message)
    {
        if (!(bool)$id_order) {
            return false;
        }

        $new_message = new Message();
        $message = str_replace("<br>", "\n", $message);

        if (!Validate::isCleanHtml($message)) {
            $message = $this->l('Payment message is not valid, please check your module.');
        }

        $new_message->message = $message;
        $new_message->id_order = (int)$id_order;
        $new_message->private = 1;

        return $new_message->add();
    }

    /******** RENDER FORM - REQUIRED SETTINGS ********/
    public function renderFormGeneral()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => '&nbsp;'.$this->l('Main module settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'hidden', /* switch / hidden */
                        'label' => $this->l('Allow Debugging mode'),
                        'desc' => $this->l('Use this option for testing purposes only. Must be set to "No" in production environment!'),
                        'name' => 'TRUSTPAY_DEBUG',
                        'values' => array(
                            array(
                                'id' => 'debug_on',
                                'value' => '1',
                                'label' => $this->l('Yes'),
                            ),
                            array(
                                'id' => 'debug_off',
                                'value' => '0',
                                'label' => $this->l('No'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Merchant account ID').' ('.$this->l('AID').')',
                        'name' => 'TRUSTPAY_AID',
                        'required' => true,
                        'maxlength' => 10,
                        'desc' => $this->l('Your unique Merchant account ID you provided by TrustPay.'),
                        'prefix' => '<i class="icon-user"></i>',
                        'class' => 'fixed-width-xxl',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Secret key'),
                        'name' => 'TRUSTPAY_PWD',
                        'required' => true,
                        'maxlength' => 200,
                        'desc' => $this->l('Your unique Secret key for encryption purposes you provided by TrustPay.'),
                        'prefix' => '<i class="icon-shield"></i>',
                        'class' => 'fixed-width-xxl',
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Account currency type'),
                        'name' => 'TRUSTPAY_CUR',
                        'required' => true,
                        'class' => 'fixed-width-xxl',
                        'options' => array(
                            'query' => array(
                                array('id' => '', 'name' => $this->l('-- choose (required) --')),
                                array('id' => 'EUR', 'name' => $this->l('Euro')),
                                array('id' => 'CZK', 'name' => $this->l('Czech koruna')),
/*
                                array('id' => 'BGN', 'name' => $this->l('Bulgarian lev')),
                                array('id' => 'DKK', 'name' => $this->l('Danish krone')),
                                array('id' => 'GBP', 'name' => $this->l('Pound Sterling')),
                                array('id' => 'HUF', 'name' => $this->l('Forint')),
                                array('id' => 'NOK', 'name' => $this->l('Norwegian krone')),
                                array('id' => 'RON', 'name' => $this->l('Romanian new leu')),
                                array('id' => 'TRY', 'name' => $this->l('Turkish lira')),
                                array('id' => 'USD', 'name' => $this->l('US Dollar')),
                                array('id' => 'PLN', 'name' => $this->l('Zloty')),
                                array('id' => 'BAM', 'name' => $this->l('Bosnia and Herzegovina convertible mark')),
                                array('id' => 'HRK', 'name' => $this->l('Croatian Kuna')),
                                array('id' => 'RSD', 'name' => $this->l('Serbian dinar')),
                                array('id' => 'LVL', 'name' => $this->l('Latvian lats')),
                                array('id' => 'LTL', 'name' => $this->l('Lithuanian litas')),
*/
                                    ),
                                    'id' => 'id',
                                    'name' => 'name',
                                    ),
                                ),
                   array(
                        'type' => 'select',
                        'label' => $this->l('Interface language'),
                        'name' => 'TRUSTPAY_LNG',
                        'required' => false,
                        'class' => 'fixed-width-xxl',
                        'options' => array(
                            'query' => array(
                                array('id' => '', 'name' => $this->l('-- choose --')),
                                array('id' => 'bg', 'name' => $this->l('Bulgarian')),
                                array('id' => 'bs', 'name' => $this->l('Bosnian')),
                                array('id' => 'cs', 'name' => $this->l('Czech')),
                                array('id' => 'de', 'name' => $this->l('German')),
                                array('id' => 'en', 'name' => $this->l('English')),
                                array('id' => 'es', 'name' => $this->l('Spanish')),
                                array('id' => 'et', 'name' => $this->l('Estonian')),
                                array('id' => 'hr', 'name' => $this->l('Croatian')),
                                array('id' => 'hu', 'name' => $this->l('Hungarian')),
                                array('id' => 'it', 'name' => $this->l('Italian')),
                                array('id' => 'lt', 'name' => $this->l('Lithuanian')),
                                array('id' => 'lv', 'name' => $this->l('Latvian')),
                                array('id' => 'pl', 'name' => $this->l('Polish')),
                                array('id' => 'ro', 'name' => $this->l('Romanian')),
                                array('id' => 'ru', 'name' => $this->l('Russian')),
                                array('id' => 'sk', 'name' => $this->l('Slovak')),
                                array('id' => 'sl', 'name' => $this->l('Slovene')),
                                array('id' => 'sr', 'name' => $this->l('Serbian')),
                                array('id' => 'tr', 'name' => $this->l('Turkish')),
                                array('id' => 'uk', 'name' => $this->l('Ukrainian')),
                                    ),
                                    'id' => 'id',
                                    'name' => 'name',
                                    ),
                                ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Merchant country'),
                        'name' => 'TRUSTPAY_CNT',
                        'required' => false,
                        'class' => 'fixed-width-xxl',
                        'options' => array(
                            'query' => array(
                                array('id' => '', 'name' => $this->l('-- choose --')),
                                array('id' => 'SK', 'name' => $this->l('Slovak Republic')),
                                array('id' => 'CZ', 'name' => $this->l('Czech Republic')),
/*
                                array('id' => 'BG', 'name' => $this->l('Bulgaria')),
                                array('id' => 'DK', 'name' => $this->l('Denmark')),
                                array('id' => 'EE', 'name' => $this->l('Estonia')),
                                array('id' => 'FI', 'name' => $this->l('Finland')),
                                array('id' => 'GR', 'name' => $this->l('Greece')),
                                array('id' => 'HU', 'name' => $this->l('Hungary')),
                                array('id' => 'LT', 'name' => $this->l('Lithuania')),
                                array('id' => 'LV', 'name' => $this->l('Latvia')),
                                array('id' => 'NO', 'name' => $this->l('Norway')),
                                array('id' => 'RO', 'name' => $this->l('Romania')),
                                array('id' => 'SI', 'name' => $this->l('Slovenia')),
                                array('id' => 'TR', 'name' => $this->l('Turkey')),
                                array('id' => 'PL', 'name' => $this->l('Poland')),
                                array('id' => 'BA', 'name' => $this->l('Bosnia and Herzegovina')),
                                array('id' => 'HR', 'name' => $this->l('Croatia')),
                                array('id' => 'RS', 'name' => $this->l('Serbia')),
*/
                                    ),
                                    'id' => 'id',
                                    'name' => 'name',
                                    ),
                                ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Payment description'),
                        'desc' => $this->l('Max. 140 characters. Diacritical marks not allowed.').'<br>'.$this->l('If you leave blank, Cart ID will be filled.'),
                        'name' => 'TRUSTPAY_DSC',
                        'required' => false,
                        'maxlength' => 140,
                        'prefix' => '<i class="icon-align-justify"></i>',
                        'class' => 'fixed-width-xxl',
                    ),
                    array(
                        'type'   => 'radio', 
                        'label'  => $this->l('Online bank & internal transfers'),
                        'desc'   => '<span class="'.$this->name.'-desc">'.$this->l('Enable this option only if you have an agreement with TrustPay for domestic bank wire payments. Otherwise an error message will be shown to all customers. Please ask TrustPay company to enable bankwire/transfer payments for you.').'</span>',
                        'name'   => 'TRUSTPAY_PAYURL',
                        'required'  => false,
                        'class'    => 't',
                        'values'   => array( 
                            array(
                                'id'    => 'live',
                                'value' => $this->url_wire_live,
                                    'label' => '<strong>'.$this->l('enabled - Live mode').'</strong>',
                            ),
                            array(
                                'id'    => 'sandbox',
                                'value' => $this->url_wire_test,
                                    'label' => '<strong>'.$this->l('enabled - Testing mode (Sandbox)').'</strong>',
                            ),
                            array(
                            'id'    => 'disable',
                            'value' => 'disable',
                                    'label' => '<strong>'.$this->l('disabled').'</strong>',
                        ),
                      ),
                    ),
                    array(
                        'type'   => 'radio', 
                        'label'  => $this->l('TrustCard payments'),
                        'desc'   => '<span class="'.$this->name.'-desc">'.$this->l('Enable this option only if you have an agreement with TrustPay for card payments. Otherwise an error message will be shown to all customers. Please ask TrustPay company to enable card payments for you.').'</span>',
                        'name'   => 'TRUSTPAY_PAYURLCARD',
                        'required'  => false,
                        'class'    => 't',
                        'values'   => array( 
                            array(
                                'id'    => 'livetc',
                                'value' => $this->url_card_live,
                                'label' => '<strong>'.$this->l('enabled - Live mode').'</strong>',
                            ),
                            array(
                                'id'    => 'sandboxtc',
                                'value' => $this->url_card_test,
                                'label' => '<strong>'.$this->l('enabled - Testing mode (Sandbox)').'</strong>',
                            ),
                            array(
                            'id'    => 'disabletc',
                            'value' => 'disable',
                                'label' => '<strong>'.$this->l('disabled').'</strong>',
                        ),
                      ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }

    /******** RENDER FORM - ORDER SETTINGS ********/
    public function renderFormOrder()
    {
        $statuses = OrderState::getOrderStates($this->context->language->id);

        /* AWAITING status */
        $statuses_awaiting_list = array(
            0 => array(
                'id' => 0,
                'name' => $this->l('-- select an order status --'),
            )
        );

        foreach ($statuses as $status_awaiting) {
            $statuses_awaiting_list[] = array(
                'id' => $status_awaiting['id_order_state'],
                'name' => $status_awaiting['id_order_state'] == Configuration::get('TRUSTPAY_OS_AWAITING') ? $status_awaiting['name'].' &nbsp; [*]' : $status_awaiting['name'],
            );
        }

        /* CARDWAITING status */
        $statuses_cardwaiting_list = array(
            0 => array(
                'id' => 0,
                'name' => $this->l('-- select an order status --'),
            )
        );

        foreach ($statuses as $status_cardwaiting) {
            $statuses_cardwaiting_list[] = array(
                'id' => $status_cardwaiting['id_order_state'],
                'name' => $status_cardwaiting['id_order_state'] == Configuration::get('TRUSTPAY_OS_CARDWAITING') ? $status_cardwaiting['name'].' &nbsp; [*]' : $status_cardwaiting['name'],
            );
        }

        /* PENDING status */
        $statuses_pending_list = array(
            0 => array(
                'id' => 0,
                'name' => $this->l('-- select an order status --'),
            )
        );

        foreach ($statuses as $status_pending) {
            $statuses_pending_list[] = array(
                'id' => $status_pending['id_order_state'],
                'name' => $status_pending['id_order_state'] == Configuration::get('TRUSTPAY_OS_PENDING') ? $status_pending['name'].' &nbsp; [*]' : $status_pending['name'],
            );
        }

        /* ANNOUNCED status */
        $statuses_announced_list = array(
            0 => array(
                'id' => 0,
                'name' => $this->l('-- select an order status --'),
            )
        );

        foreach ($statuses as $status_announced) {
            $statuses_announced_list[] = array(
                'id' => $status_announced['id_order_state'],
                'name' => $status_announced['id_order_state'] == Configuration::get('TRUSTPAY_OS_ANNOUNCED') ? $status_announced['name'].' &nbsp; [*]' : $status_announced['name'],
            );
        }

        /* AUTHORIZED status */
        $statuses_authorized_list = array(
            0 => array(
                'id' => 0,
                'name' => $this->l('-- select an order status --'),
            )
        );

        foreach ($statuses as $status_authorized) {
            $statuses_authorized_list[] = array(
                'id' => $status_authorized['id_order_state'],
                'name' => $status_authorized['id_order_state'] == Configuration::get('TRUSTPAY_OS_AUTHORIZED') ? $status_authorized['name'].' &nbsp; [*]' : $status_authorized['name'],
            );
        }

        /* PROCESSING status */
        $statuses_processing_list = array(
            0 => array(
                'id' => 0,
                'name' => $this->l('-- select an order status --'),
            )
        );

        foreach ($statuses as $status_processing) {
            $statuses_processing_list[] = array(
                'id' => $status_processing['id_order_state'],
                'name' => $status_processing['id_order_state'] == Configuration::get('TRUSTPAY_OS_PROCESSING') ? $status_processing['name'].' &nbsp; [*]' : $status_processing['name'],
            );
        }

        /* AUTHORIZEDONLY status */
        $statuses_authorizedonly_list = array(
            0 => array(
                'id' => 0,
                'name' => $this->l('-- select an order status --'),
            )
        );

        foreach ($statuses as $status_authorizedonly) {
            $statuses_authorizedonly_list[] = array(
                'id' => $status_authorizedonly['id_order_state'],
                'name' => $status_authorizedonly['id_order_state'] == Configuration::get('TRUSTPAY_OS_AUTHORIZEDONLY') ? $status_authorizedonly['name'].' &nbsp; [*]' : $status_authorizedonly['name'],
            );
        }

        /* PAYMENT RECEIVED status */
        $statuses_ok_list = array(
            0 => array(
                'id' => 0,
                'name' => $this->l('-- select an order status --'),
            )
        );

        foreach ($statuses as $status) {
            $statuses_ok_list[] = array(
                'id' => $status['id_order_state'],
                'name' => $status['id_order_state'] == 2 ? $status['name'].' &nbsp; [*]' : $status['name'],
            );
        }

        /* PAYMENT ERROR status */
        $statuses_fail_list = array(
            0 => array(
                'id' => 0,
                'name' => $this->l('-- select an order status --'),
            )
        );

        foreach ($statuses as $status_fail) {
            $statuses_fail_list[] = array(
                'id' => $status_fail['id_order_state'],
                'name' => $status_fail['id_order_state'] == 8 ? $status_fail['name'].' &nbsp; [*]' : $status_fail['name'],
            );
        }

        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => '&nbsp;'.$this->l('Order settings'),
                    'icon' => 'icon-flag',
                ),
                'input' => array(
                    array(
                        'type'   => 'hidden', /* Radio - TO-DO - doprogramovat notification.php */
                        'label'  => $this->l('Order method'),
                        'name'   => 'TRUSTPAY_ORDER_METHOD',
                        'required'  => true,
                        'class'    => 't',
                        'values'   => array( 
                            array(
                                'id'    => 'before',
                                'value' => 'before',
                                'label' => '<strong>'.$this->l('place an order before the payment').'</strong><br>'.$this->l('(the transaction identifier is Order ID)'),
                            ),
                            // TO-DO - doprogramovat notification.php
                            array(
                                'id'    => 'after',
                                'value' => 'after',
                                'label' => '<strong>'.$this->l('place an order after the payment').'</strong><br>'.$this->l('(the transaction identifier is Cart ID)'),
                            ),
                        ),
                    ),
                    array(
                        'type'   => 'hidden', /* radio */
                        'label'  => $this->l('Checkout type'),
                        'name'   => 'TRUSTPAY_CHECKOUT_TYPE',
                        'required'  => true,
                        'class'    => 't',
                        'values'   => array( 
                            array(
                                'id'    => 'type_checkout',
                                'value' => 'checkout',
                                'label' => '<strong>'.$this->l('Checkout inline').'</strong><br>'.$this->l('After clicking on "Order with an obligation to pay" button the popup window with payment options will show up.'),
                            ),
                            array(
                                'id'    => 'type_page',
                                'value' => 'page',
                                'label' => '<strong>'.$this->l('Single payment page').'</strong><br>'.$this->l('After clicking on "Order with an obligation to pay" button the summary page will be displayed. Then the customer can click and open popup window with payment options.').'<br><br>',
                            ),
                        ),
                    ),
                    array(
                        'type'   => 'hidden', /* TO-DO */
                        'label'  => $this->l('Create order'),
                        'name'   => 'TRUSTPAY_ORDER_ALWAYS',
                        'required'  => true,
                        'class'    => 't',
                        'values'   => array( 
                            array(
                                'id'    => 'order_always',
                                'value' => 'always',
                                'label' => '<strong>'.$this->l('Always').'</strong><span class="'.$this->name.'-desc">'.$this->l('PrestaShop will be create an order always when customer try to made payment with TrustPay gateway. No matter if the payment will be successful or not.').'</span>',
                            ),
                            array(
                                'id'    => 'order_successful',
                                'value' => 'successful_only',
                                'label' => '<strong>'.$this->l('Successful payment only').'</strong><span class="'.$this->name.'-desc">'.$this->l('PrestaShop will be create an order after successful payment only. Unsuccessuful payment return customer into the checkoutand no order will be created.').'</span><br><br>',
                            ),
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Awaiting payment order status'),
                        'hint' => $this->l('Default payment status is "Awaiting TrustPay payment".'),
                        'name' => 'TRUSTPAY_STATUS_AWAITING',
                        'required' => true,
                        'default_value' => Configuration::get('TRUSTPAY_OS_AWAITING'),
                        'class' => 'fixed-width-xxl',
                        'options' => array(
                            'query' => $statuses_awaiting_list,
                            'id' => 'id',
                            'name' => 'name',
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Awaiting card payment order status'),
                        'hint' => $this->l('Default payment status is "Awaiting TrustCard payment".'),
                        'name' => 'TRUSTPAY_STATUS_CARDWAITING',
                        'required' => true,
                        'default_value' => Configuration::get('TRUSTPAY_OS_CARDWAITING'),
                        'class' => 'fixed-width-xxl',
                        'options' => array(
                            'query' => $statuses_cardwaiting_list,
                            'id' => 'id',
                            'name' => 'name',
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Awaiting offline payment order status'),
                        'hint' => $this->l('Default payment status is "Awaiting TrustPay offline payment".'),
                        'name' => 'TRUSTPAY_STATUS_PENDING',
                        'required' => true,
                        'default_value' => Configuration::get('TRUSTPAY_OS_PENDING'),
                        'class' => 'fixed-width-xxl',
                        'options' => array(
                            'query' => $statuses_pending_list,
                            'id' => 'id',
                            'name' => 'name',
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Announced payment order status'),
                        'hint' => $this->l('Default payment status is "TrustPay payment announced".'),
                        'name' => 'TRUSTPAY_STATUS_ANNOUNCED',
                        'required' => true,
                        'default_value' => Configuration::get('TRUSTPAY_OS_ANNOUNCED'),
                        'class' => 'fixed-width-xxl',
                        'options' => array(
                            'query' => $statuses_announced_list,
                            'id' => 'id',
                            'name' => 'name',
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Authorized payment order status'),
                        'hint' => $this->l('Default payment status is "TrustPay payment authorized".'),
                        'name' => 'TRUSTPAY_STATUS_AUTHORIZED',
                        'required' => true,
                        'default_value' => Configuration::get('TRUSTPAY_OS_AUTHORIZED'),
                        'class' => 'fixed-width-xxl',
                        'options' => array(
                            'query' => $statuses_authorized_list,
                            'id' => 'id',
                            'name' => 'name',
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Processing payment order status'),
                        'hint' => $this->l('Default payment status is "TrustPay payment is processing".'),
                        'name' => 'TRUSTPAY_STATUS_PROCESSING',
                        'required' => true,
                        'default_value' => Configuration::get('TRUSTPAY_OS_PROCESSING'),
                        'class' => 'fixed-width-xxl',
                        'options' => array(
                            'query' => $statuses_processing_list,
                            'id' => 'id',
                            'name' => 'name',
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Card authorized payment order status'),
                        'hint' => $this->l('Default payment status is "Card payment authorized".'),
                        'name' => 'TRUSTPAY_STATUS_AUTHORIZEDONLY',
                        'required' => true,
                        'default_value' => Configuration::get('TRUSTPAY_OS_AUTHORIZEDONLY'),
                        'class' => 'fixed-width-xxl',
                        'options' => array(
                            'query' => $statuses_authorizedonly_list,
                            'id' => 'id',
                            'name' => 'name',
                        ),
                    ),
                    /* Build-in payment states */
                    array(
                        'type' => 'select',
                        'label' => $this->l('Success payment order status'),
                        'hint' => $this->l('Default payment status is "Payment accepted".'),
                        'name' => 'TRUSTPAY_STATUS_OK',
                        'required' => true,
                        'default_value' => '2',
                        'class' => 'fixed-width-xxl',
                        'options' => array(
                            'query' => $statuses_ok_list,
                            'id' => 'id',
                            'name' => 'name',
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Error payment order status'),
                        'hint' => $this->l('Default payment status is "Payment error".'),
                        'name' => 'TRUSTPAY_STATUS_FAIL',
                        'required' => true,
                        'default_value' => '8',
                        'class' => 'fixed-width-xxl',
                        'options' => array(
                            'query' => $statuses_fail_list,
                            'id' => 'id',
                            'name' => 'name',
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmitOrder';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }

    /******** RENDER FORM - DESIGN SETTINGS ********/
    public function renderFormIcon()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => '&nbsp;'.$this->l('Design settings'),
                    'icon' => 'icon-paint-brush',
                ),
                'input' => array(
                    array(
                        'type'   => 'radio',
                        'label'  => $this->l('What theme are you using?'),
                        'desc'   => '<span class="'.$this->name.'-desc">'.$this->l('The module using Font Awesome Icons the same as built-in "default-bootstrap" PrestaShop theme. The other theme can using other icon set so Font Awesome Icons wouldn\'t have to render correctly. This option will disable Font Awesome Icons if your current theme not use them.'),
                        'name'   => 'TRUSTPAY_THEME',
                        'required'  => false,
                        'class'    => 't',
                        'values'   => array( 
                            array(
                                'id'    => 'default',
                                'value' => 'default',
                                'label' => '<strong>'.$this->l('Built-in "default-bootstrap" theme').'</strong>',
                            ),
                            array(
                                'id'    => 'thirdparty',
                                'value' => 'thirdparty',
                                'label' => '<strong>'.$this->l('Other theme').'</strong>',
                            ),
                        ),
                    ),
                    array(
                        'type'   => 'radio', 
                        'label'  => $this->l('Payment icon(s)'),
                        'desc'   => '<div class="'.$this->name.'-icon-130">
                                    <img src="'.$this->logo_tp.'" alt="'.$this->l('Logo:').' '.$this->l('TrustPay').'">
                                    <img src="'.$this->logo_tc.'" alt="'.$this->l('Logo:').' '.$this->l('TrustCard').'">
                                    </div>',
                        'name'   => 'TRUSTPAY_ICON',
                        'required'  => false,
                        'class'    => 't',
                        'values'   => array( 
                            array(
                                'id'    => 'no',
                                'value' => '0',
                                'label' => '<strong>'.$this->l('not display').'</strong>',
                            ),
                            array(
                                'id'    => 'product',
                                'value' => 'p',
                                'label' => '<strong>'.$this->l('display in product detail').'</strong>',
                            ),
                            array(
                                'id'    => 'left',
                                'value' => 'l',
                                'label' => '<strong>'.$this->l('display in left column').'</strong>',
                            ),
                            array(
                            'id'    => 'right',
                            'value' => 'r',
                                'label' => '<strong>'.$this->l('display in right column').'</strong>',
                        ),
                      ),
                    ),
                        array(
                        'type'   => 'switch', 
                        'label'  => $this->l('Show domestic online payment systems logos'),
                        'hint'  => $this->l('Show available domestic online payment systems along with TrustPay payment on checkout page.'),
                        'desc'   => '<div class="'.$this->name.'-icon-90">
                                    <img src="'.$this->logo_tb.'" alt="'.$this->l('Logo:').' '.$this->l('Tatra banka').'">
                                    <img src="'.$this->logo_slsp.'" alt="'.$this->l('Logo:').' '.$this->l('Slovenska sporitelna').'">
                                    <img src="'.$this->logo_vub.'" alt="'.$this->l('Logo:').' '.$this->l('VUB').'">
                                    <img src="'.$this->logo_pb.'" alt="'.$this->l('Logo:').' '.$this->l('Postova banka').'">
                                    <img src="'.$this->logo_csob.'" alt="'.$this->l('Logo:').' '.$this->l('CSOB').'">
                                    <img src="'.$this->logo_ib.'" alt="'.$this->l('Logo:').' '.$this->l('Other bank').'">
                                    </div>',                        
                        'name'   => 'TRUSTPAY_BANKICON',
                        'required'  => false,
                        'class'    => 't',
                        'values' => array(
                            array(
                                'id' => 'bankicon_on',
                                'value' => '1',
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'bankicon_off',
                                'value' => '0',
                                'label' => $this->l('Disabled'),
                            ),
                      ),
                    ),
                        array(
                        'type'   => 'switch', 
                        'label'  => $this->l('Show accepted card logos'),
                        'hint'  => $this->l('Show VISA, VISA Electron, MasterCard and Maestro logos along with TrustCard payment on checkout page.'),
                        'desc'   => '<div class="'.$this->name.'-icon-90">
                                    <img src="'.$this->logo_visa.'" alt="'.$this->l('Logo:').' '.$this->l('VISA').'">
                                    <img src="'.$this->logo_visa_electron.'" alt="'.$this->l('Logo:').' '.$this->l('VISA Electron').'">
                                    <img src="'.$this->logo_mastercard.'" alt="'.$this->l('Logo:').' '.$this->l('MasterCard').'">
                                    <img src="'.$this->logo_maestro.'" alt="'.$this->l('Logo:').' '.$this->l('Maestro').'">
                                    <img src="'.$this->logo_apple_pay.'" alt="'.$this->l('Logo:').' '.$this->l('Apple Pay').'">
                                    </div>',
                        'name'   => 'TRUSTPAY_CARDICON',
                        'required'  => false,
                        'class'    => 't',
                        'values' => array(
                            array(
                                'id' => 'cardicon_on',
                                'value' => '1',
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'cardicon_off',
                                'value' => '0',
                                'label' => $this->l('Disabled'),
                            ),
                      ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Payment icon destination CMS page'),
                        'hint'  => $this->l('Select CMS page where will follow link from the icon.'),
                        'name' => 'TRUSTPAY_CMS_ID',
                        'required' => false,
                        'default_value' => '3',
                        'options' => array(
                            'query' => CMS::listCms($this->context->language->id),
                            'id' => 'id_cms',
                            'name' => 'meta_title',
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmitIcon';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }

    /******** RENDER FORM - PRIVATE NOTEST ********/
    public function renderFormPrivatenotes()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => '&nbsp;'.$this->l('Private notes'),
                    'icon' => 'icon-file-alt',
                ),
                'input' => array(
                    array(
                        'type' => 'textarea',
                        'label' => $this->l('Private notes'),
                        'name' => 'TRUSTPAY_PRIVATE_NOTES',
                        'required' => false,
                        'desc' => $this->l('Here you can store your various notes e.g. contact info on TrustPay technical support, creditials into TrustPay merchant center etc.'),
                        'rows' => '10',
                        'col' => '5',
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmitPrivate';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }

    /******** GET FIELD VALUES ********/
    public function getConfigFieldsValues()
    {
        return array(
            'TRUSTPAY_DEBUG' => Tools::getValue('TRUSTPAY_DEBUG', Configuration::get('TRUSTPAY_DEBUG')),
            'TRUSTPAY_AID' => Tools::getValue('TRUSTPAY_AID', Configuration::get('TRUSTPAY_AID')),
            'TRUSTPAY_PWD' => Tools::getValue('TRUSTPAY_PWD', Configuration::get('TRUSTPAY_PWD')),
            'TRUSTPAY_CUR' => Tools::getValue('TRUSTPAY_CUR', Configuration::get('TRUSTPAY_CUR')),
            'TRUSTPAY_LNG' => Tools::getValue('TRUSTPAY_LNG', Configuration::get('TRUSTPAY_LNG')),
            'TRUSTPAY_CNT' => Tools::getValue('TRUSTPAY_CNT', Configuration::get('TRUSTPAY_CNT')),
            'TRUSTPAY_DSC' => Tools::getValue('TRUSTPAY_DSC', Configuration::get('TRUSTPAY_DSC')),
            'TRUSTPAY_PAYURL' => Tools::getValue('TRUSTPAY_PAYURL', Configuration::get('TRUSTPAY_PAYURL')),
            'TRUSTPAY_PAYURLCARD' => Tools::getValue('TRUSTPAY_PAYURLCARD', Configuration::get('TRUSTPAY_PAYURLCARD')),

            'TRUSTPAY_ORDER_METHOD' => Tools::getValue('TRUSTPAY_ORDER_METHOD', Configuration::get('TRUSTPAY_ORDER_METHOD')),
            'TRUSTPAY_CHECKOUT_TYPE' => Tools::getValue('TRUSTPAY_CHECKOUT_TYPE', Configuration::get('TRUSTPAY_CHECKOUT_TYPE')),
            'TRUSTPAY_ORDER_ALWAYS' => Tools::getValue('TRUSTPAY_ORDER_ALWAYS', Configuration::get('TRUSTPAY_ORDER_ALWAYS')),
            'TRUSTPAY_STATUS_AWAITING' => Tools::getValue('TRUSTPAY_STATUS_AWAITING', Configuration::get('TRUSTPAY_STATUS_AWAITING')),
            'TRUSTPAY_STATUS_CARDWAITING' => Tools::getValue('TRUSTPAY_STATUS_CARDWAITING', Configuration::get('TRUSTPAY_STATUS_CARDWAITING')),

            'TRUSTPAY_STATUS_PENDING' => Tools::getValue('TRUSTPAY_STATUS_PENDING', Configuration::get('TRUSTPAY_STATUS_PENDING')),
            'TRUSTPAY_STATUS_ANNOUNCED' => Tools::getValue('TRUSTPAY_STATUS_ANNOUNCED', Configuration::get('TRUSTPAY_STATUS_ANNOUNCED')),
            'TRUSTPAY_STATUS_AUTHORIZED' => Tools::getValue('TRUSTPAY_STATUS_AUTHORIZED', Configuration::get('TRUSTPAY_STATUS_AUTHORIZED')),
            'TRUSTPAY_STATUS_PROCESSING' => Tools::getValue('TRUSTPAY_STATUS_PROCESSING', Configuration::get('TRUSTPAY_STATUS_PROCESSING')),
            'TRUSTPAY_STATUS_AUTHORIZEDONLY' => Tools::getValue('TRUSTPAY_STATUS_AUTHORIZEDONLY', Configuration::get('TRUSTPAY_STATUS_AUTHORIZEDONLY')),

            'TRUSTPAY_STATUS_OK' => Tools::getValue('TRUSTPAY_STATUS_OK', Configuration::get('TRUSTPAY_STATUS_OK')),
            'TRUSTPAY_STATUS_FAIL' => Tools::getValue('TRUSTPAY_STATUS_FAIL', Configuration::get('TRUSTPAY_STATUS_FAIL')),

            'TRUSTPAY_THEME' => Tools::getValue('TRUSTPAY_THEME', Configuration::get('TRUSTPAY_THEME')),
            'TRUSTPAY_ICON' => Tools::getValue('TRUSTPAY_ICON', Configuration::get('TRUSTPAY_ICON')),
            'TRUSTPAY_BANKICON' => Tools::getValue('TRUSTPAY_BANKICON', Configuration::get('TRUSTPAY_BANKICON')),
            'TRUSTPAY_CARDICON' => Tools::getValue('TRUSTPAY_CARDICON', Configuration::get('TRUSTPAY_CARDICON')),
            'TRUSTPAY_CMS_ID' => Tools::getValue('TRUSTPAY_CMS_ID', Configuration::get('TRUSTPAY_CMS_ID')),

            'TRUSTPAY_PRIVATE_NOTES' => Tools::getValue('TRUSTPAY_PRIVATE_NOTES', Configuration::get('TRUSTPAY_PRIVATE_NOTES')),
        );
    }

    /******** NOTIFICATION ********/
    public function _notifyMe()
    {
        $this->authormail = 'addons@caleydon.com';
        $data = array("{ps_version}" => "PrestaShop "._PS_VERSION_, "{ps_email}" => Configuration::get('PS_SHOP_EMAIL'), "{ps_url}" => $_SERVER['SERVER_NAME'], "{mod_name}" => $this->displayName, "{mod_version}" => $this->version, "{admin_name}" => $this->context->employee->firstname.' '.$this->context->employee->lastname, "{admin_email}" => $this->context->employee->email, "{date}" => date(d).'.'.date(m).'.'.date(Y).', '.date(H).':'.date(i).':'.date(s)
        );

        Mail::Send(Configuration::get('PS_LANG_DEFAULT'), 'notification', 'MOD: '.$this->displayName.' '.$this->version, $data, $this->authormail, null, null, null, null, null, _PS_MODULE_DIR_.$this->name.'/mails/', false, $this->context->shop->id);
    }

    /******** MODULE OFFER ********/
    private function _displayOffer()
    {
        return $this->display(__FILE__, 'mod_offer.tpl');
    }

    /******** CSS: Front Office ********/
    public function hookDisplayHeader($params)
    {
        $this->context->controller->addCSS($this->_path.'views/css/styles.css', 'all');
    }

    /******** CSS: Back Office ********/
    public function hookDisplayBackOfficeHeader()
    {
        $this->context->controller->addCSS($this->_path.'views/css/styles-bo.css', 'all');
        $this->context->controller->addJS($this->_path.'views/js/functions-bo.js', 'all');
    }

    /******** ICON TEMPLATE VARIABLES ********/
    public function getIconVariables()
    {
        return array(
            'module_name' => $this->displayName,
            'module_filename' => $this->name,
            'icon_position' => Configuration::get('TRUSTPAY_ICON'),
            'cms_link' => $this->context->link->getCMSLink((int)Configuration::get('TRUSTPAY_CMS_ID')),

            'bankwirepayment_status' => $this->PAYURL,
            'trustcard_status' => $this->PAYURLCARD,
            'icon_cards' => $this->CARDICON,

            'logo_tp' => $this->logo_tp,
            'logo_tb' => $this->logo_tb,
            'logo_slsp' => $this->logo_slsp,
            'logo_vub' => $this->logo_vub,
            'logo_pb' => $this->logo_pb,
            'logo_csob' => $this->logo_csob,
            'logo_ib' => $this->logo_ib,

            'logo_tc' => $this->logo_tc,
            'logo_visa' => $this->logo_visa,
            'logo_visa_electron' => $this->logo_visa_electron,
            'logo_mastercard' => $this->logo_mastercard,
            'logo_maestro' => $this->logo_maestro,
            'logo_apple_pay' => $this->logo_apple_pay,

            'logo_alt_tp' => $this->l('Logo:').' '.$this->l('TrustPay'),
            'logo_alt_tb' => $this->l('Logo:').' '.$this->l('Tatra banka'),
            'logo_alt_slsp' => $this->l('Logo:').' '.$this->l('Slovenska sporitelna'),
            'logo_alt_vub' => $this->l('Logo:').' '.$this->l('VUB'),
            'logo_alt_pb' => $this->l('Logo:').' '.$this->l('Postova banka'),
            'logo_alt_csob' => $this->l('Logo:').' '.$this->l('CSOB'),
            'logo_alt_ib' => $this->l('Logo:').' '.$this->l('Other bank'),

            'logo_alt_tc' => $this->l('Logo:').' '.$this->l('TrustCard'),
            'logo_alt_visa' => $this->l('Logo:').' '.$this->l('VISA'),
            'logo_alt_visa_electron' => $this->l('Logo:').' '.$this->l('VISA Electron'),
            'logo_alt_mastercard' => $this->l('Logo:').' '.$this->l('MasterCard'),
            'logo_alt_maestro' => $this->l('Logo:').' '.$this->l('Maestro'),
            'logo_alt_apple_pay' => $this->l('Logo:').' '.$this->l('Apple Pay'),

            'this_path' => $this->_path,
            'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/',
        );
    }

    /******** PAYMENT ICON - LEFT COLUMN ********/
    public function hookLeftColumn($params)
    {
        $this->context->smarty->assign(
            $this->getIconVariables()
        );

        return $this->display(__FILE__, 'icon_left.tpl');
    }

    /******** PAYMENT ICON - RIGHT COLUMN ********/
    public function hookRightColumn($params)
    {
        $this->context->smarty->assign(
            $this->getIconVariables()
        );

        return $this->display(__FILE__, 'icon_right.tpl');
    }

    /******** PAYMENT ICON - PRODUCT DETAIL ********/
    public function hookDisplayProductButtons($params)
    {
        $this->context->smarty->assign(
            $this->getIconVariables()
        );

        return $this->display(__FILE__, 'icon_product.tpl');
    }

    /******** MESSAGE NOTIFICATIONS ********/
    public function getL($key)
    {
        $translations = array(
            'msg_pm_awaiting'         => $this->l('Awaiting TrustPay domestic bank payment'),
            'msg_pm_trustcard'        => $this->l('Awaiting card payment by TrustCard'),

            'msg_noverif'             => $this->l('TrustPay payment transaction was not verified.'),
            'msg_aid'                 => $this->l('Merchant account ID (AccountId) not specified.'),
            'msg_typ'                 => $this->l('Type of transaction (Type) not specified.'),
            'msg_crdt'                => $this->l('Transaction is not credited to the account.'),
            'msg_dbit'                => $this->l('Transaction is debit.'),
            'msg_cur'                 => $this->l('Currency of the payment (Currency) not specified.'),
            'msg_pid'                 => $this->l('TrustPay Payment ID (PaymentId) not specified.'),
            'msg_oid'                 => $this->l('TrustPay Order ID (OrderId) not specified.'),
            'msg_tss'                 => $this->l('Transaction signed status (TSS) not specified.'),
            'msg_tssno'               => $this->l('The amount paid is not same as requested.'),
            'msg_ref'                 => $this->l('Merchant\'s payment identification (REF) not specified.'),
            'msg_cart'                => $this->l('Cart ID not found.'),
            'msg_order'               => $this->l('The order has been placed already.'),
            'msg_sign'                => $this->l('Security signature not specified.'),
            'msg_signs'               => $this->l('The security signs not match.'),
            'msg_total'               => $this->l('Total amount not specified.'),
            'msg_amount'              => $this->l('The order\'s and paid total amount are not equal.'),
            'msg_required'            => $this->l('Amount required:'),
            'msg_received'            => $this->l('Amount received:'),
            'msg_ref_value'           => $this->l('Reference:'),
            'msg_pid_value'           => $this->l('Transaction ID:'),
            'msg_res_value'           => $this->l('Result code:'),
            'msg_error_value'         => $this->l('Error:'),
            'msg_paymentinfo'         => $this->l('TRUSTPAY PAYMENT INFO:'),
            'msg_pidinfo'             => $this->l('Payment ID (PaymentId)'),
            'msg_oidinfo'             => $this->l('Order ID (OrderId)'),
            'msg_cardinfo'            => $this->l('CARD INFO:'),
            'msg_cardmask'            => $this->l('Masked card number'),
            'msg_cardexp'             => $this->l('Card expiration'),
            'msg_authnumber'          => $this->l('Authorization number'),
            'msg_acqresid'            => $this->l('Card Acquirer Response ID'),

            // TrustPay result codes
            'msg_transaction'         => $this->l('TrustPay payment transaction has been done.'), // Code #0
            'msg_pending'             => $this->l('TrustPay payment is pending (Offline payment).'), // Code #1
            'msg_announced'           => $this->l('TrustPay payment was successfully announced. Waiting for payment approval.'),    // Code #2
            'msg_authorized'          => $this->l('TrustPay payment was successfully authorized. Waiting for payment approval.'),    // Code #3
            'msg_processing'          => $this->l('TrustPay payment was successfully received, but it must be internally processed.'),    // Code #4
            'msg_authorizedonly'      => $this->l('Card payment was successfully authorized, but not captured.'),    // Code #5
            'msg_1001'                => $this->l('[Code #1001] Invalid request - Data sent is not properly formatted.'),
            'msg_1002'                => $this->l('[Code #1002] Unknown account - Account with specified ID was not found.'),
            'msg_1003'                => $this->l('[Code #1003] Merchant account disabled - Merchant account has been disabled.'),
            'msg_1004'                => $this->l('[Code #1004] Invalid sign - Sign of message is invalid.'),
            'msg_1005'                => $this->l('[Code #1005] User cancel - User has cancelled payment.'),
            'msg_1006'                => $this->l('[Code #1006] Invalid authentication - The request was not properly authenticated.'),
            'msg_1007'                => $this->l('[Code #1007] Disposable balance - Requested transaction amount is greater than disposable balance.'),
            'msg_1008'                => $this->l('[Code #1008] Service not allowed - Service cannot be used or permission to use given service has not been granted.'),
            'msg_1009'                => $this->l('[Code #1009] PaySafeCard timeout - Cards allocation will be cancelled.'),
            'msg_1010'                => $this->l('[Code #1010] Transaction not found - Transaction with specified ID was not found.'),
            'msg_1011'                => $this->l('[Code #1011] Unsupported transaction - The requested action is not supported for the transaction.'),
            'msg_1014'                => $this->l('[Code #1014] Transaction was rejected by issuer.'),
            'msg_1100'                => $this->l('[Code #1100] General Error - Internal error has occurred.'),
            'msg_1101'                => $this->l('[Code #1101] Unsupported currency conversion - Currency conversion for requested currencies is not supported.'),
            'tax_incl'                => $this->l('tax incl.'),
            'order_nbr'               => $this->l('#'),
            'img_alt_banks'           => $this->l('Logo:').' '.$this->l('Domestic banks'),
            'img_alt_tc'              => $this->l('Logo:').' '.$this->l('TrustCard'),
            'img_alt_visa'            => $this->l('Logo:').' '.$this->l('VISA'),
            'img_alt_visa_electron'   => $this->l('Logo:').' '.$this->l('VISA Electron'),
            'img_alt_mastercard'      => $this->l('Logo:').' '.$this->l('MasterCard'),
            'img_alt_maestro'         => $this->l('Logo:').' '.$this->l('Maestro'),
            'img_alt_apple_pay'       => $this->l('Logo:').' '.$this->l('Apple Pay'),
            );

        return $translations[$key];
    }
}
/* EOF */